@@ -3,61 +3,90 @@ from __future__ import annotations
 import typing
 from collections import defaultdict
 from enum import Enum
-from typing import Dict
-from typing import List
 from typing import Protocol
-from typing import Set
+
+import pwnlib
+from capstone6pwndbg import *  # noqa: F403
 
 # Reverse lookup tables for debug printing
-from capstone import CS_AC
-from capstone import CS_GRP
-from capstone import CS_OP
-from capstone import *  # noqa: F403
-from capstone.arm import ARM_INS_TBB
-from capstone.arm import ARM_INS_TBH
-
-# from capstone.arm64 import ARM64_INS_B
-from capstone.arm64 import ARM64_INS_BL
-from capstone.arm64 import ARM64_INS_BLR
-from capstone.arm64 import ARM64_INS_BR
-from capstone.mips import MIPS_INS_B
-from capstone.mips import MIPS_INS_BAL
-from capstone.mips import MIPS_INS_BLTZAL
-from capstone.mips import MIPS_INS_J
-from capstone.mips import MIPS_INS_JAL
-from capstone.mips import MIPS_INS_JALR
-from capstone.mips import MIPS_INS_JR
-from capstone.ppc import PPC_INS_B
-from capstone.ppc import PPC_INS_BA
-from capstone.ppc import PPC_INS_BL
-from capstone.ppc import PPC_INS_BLA
-from capstone.riscv import RISCV_INS_C_J
-from capstone.riscv import RISCV_INS_C_JAL
-from capstone.riscv import RISCV_INS_C_JALR
-from capstone.riscv import RISCV_INS_C_JR
-from capstone.riscv import RISCV_INS_JAL
-from capstone.riscv import RISCV_INS_JALR
-from capstone.sparc import SPARC_INS_JMP
-from capstone.sparc import SPARC_INS_JMPL
-from capstone.x86 import X86_INS_JMP
-from capstone.x86 import X86Op
+from capstone6pwndbg import CS_AC
+from capstone6pwndbg import CS_GRP
+from capstone6pwndbg import CS_OP
+from capstone6pwndbg.aarch64 import AARCH64_INS_BL
+from capstone6pwndbg.aarch64 import AARCH64_INS_BLR
+from capstone6pwndbg.aarch64 import AARCH64_INS_BR
+from capstone6pwndbg.arm import ARM_INS_TBB
+from capstone6pwndbg.arm import ARM_INS_TBH
+from capstone6pwndbg.loongarch import LOONGARCH_INS_ALIAS_JR
+from capstone6pwndbg.loongarch import LOONGARCH_INS_B
+from capstone6pwndbg.loongarch import LOONGARCH_INS_BL
+from capstone6pwndbg.loongarch import LOONGARCH_INS_CALL36
+from capstone6pwndbg.loongarch import LOONGARCH_INS_JIRL
+from capstone6pwndbg.mips import MIPS_INS_ALIAS_B
+from capstone6pwndbg.mips import MIPS_INS_ALIAS_BAL
+from capstone6pwndbg.mips import MIPS_INS_B
+from capstone6pwndbg.mips import MIPS_INS_BAL
+from capstone6pwndbg.mips import MIPS_INS_BLTZAL
+from capstone6pwndbg.mips import MIPS_INS_J
+from capstone6pwndbg.mips import MIPS_INS_JAL
+from capstone6pwndbg.mips import MIPS_INS_JALR
+from capstone6pwndbg.mips import MIPS_INS_JALR_HB
+from capstone6pwndbg.mips import MIPS_INS_JR
+from capstone6pwndbg.ppc import PPC_INS_B
+from capstone6pwndbg.ppc import PPC_INS_BA
+from capstone6pwndbg.ppc import PPC_INS_BL
+from capstone6pwndbg.ppc import PPC_INS_BLA
+from capstone6pwndbg.riscv import RISCV_INS_C_J
+from capstone6pwndbg.riscv import RISCV_INS_C_JAL
+from capstone6pwndbg.riscv import RISCV_INS_C_JALR
+from capstone6pwndbg.riscv import RISCV_INS_C_JR
+from capstone6pwndbg.riscv import RISCV_INS_JAL
+from capstone6pwndbg.riscv import RISCV_INS_JALR
+from capstone6pwndbg.sparc import SPARC_INS_ALIAS_BA
+from capstone6pwndbg.sparc import SPARC_INS_ALIAS_CALL
+from capstone6pwndbg.sparc import SPARC_INS_CALL
+from capstone6pwndbg.sparc import SPARC_INS_JMPL
+from capstone6pwndbg.systemz import SYSTEMZ_INS_B
+from capstone6pwndbg.systemz import SYSTEMZ_INS_BAL
+from capstone6pwndbg.systemz import SYSTEMZ_INS_BALR
+from capstone6pwndbg.systemz import SYSTEMZ_INS_BR
+from capstone6pwndbg.systemz import SYSTEMZ_INS_BRAS
+from capstone6pwndbg.systemz import SYSTEMZ_INS_BRASL
+from capstone6pwndbg.systemz import SYSTEMZ_INS_J
+from capstone6pwndbg.systemz import SYSTEMZ_INS_JL
+from capstone6pwndbg.x86 import X86_INS_CALL
+from capstone6pwndbg.x86 import X86_INS_JMP
+from capstone6pwndbg.x86 import X86_INS_RET
+from capstone6pwndbg.x86 import X86Op
 from typing_extensions import override
 
-import pwndbg.dbg
-from pwndbg.dbg import DisassembledInstruction
+from pwndbg.dbg_mod import DisassembledInstruction
 
 # Architecture specific instructions that mutate the instruction pointer unconditionally
-# The Capstone RET and CALL groups are also used to filter CALL and RET types when we check for unconditional jumps,
-# so we don't need to manually specify those for each architecture
-UNCONDITIONAL_JUMP_INSTRUCTIONS: Dict[int, Set[int]] = {
-    CS_ARCH_X86: {X86_INS_JMP},
-    CS_ARCH_MIPS: {MIPS_INS_J, MIPS_INS_JR, MIPS_INS_JAL, MIPS_INS_JALR, MIPS_INS_BAL, MIPS_INS_B},
-    CS_ARCH_SPARC: {SPARC_INS_JMP, SPARC_INS_JMPL},
+UNCONDITIONAL_JUMP_INSTRUCTIONS: dict[int, set[int]] = {
+    CS_ARCH_X86: {X86_INS_CALL, X86_INS_RET, X86_INS_JMP},
+    CS_ARCH_MIPS: {
+        MIPS_INS_J,
+        MIPS_INS_JR,
+        MIPS_INS_JAL,
+        MIPS_INS_JALR,
+        MIPS_INS_JALR_HB,
+        MIPS_INS_BAL,
+        MIPS_INS_ALIAS_BAL,
+        MIPS_INS_B,
+        MIPS_INS_ALIAS_B,
+    },
+    CS_ARCH_SPARC: {
+        SPARC_INS_CALL,
+        SPARC_INS_ALIAS_CALL,
+        SPARC_INS_JMPL,
+        SPARC_INS_ALIAS_BA,
+    },
     CS_ARCH_ARM: {
         ARM_INS_TBB,
         ARM_INS_TBH,
     },
-    CS_ARCH_ARM64: {ARM64_INS_BL, ARM64_INS_BLR, ARM64_INS_BR},
+    CS_ARCH_AARCH64: {AARCH64_INS_BL, AARCH64_INS_BLR, AARCH64_INS_BR},
     CS_ARCH_RISCV: {
         RISCV_INS_JAL,
         RISCV_INS_JALR,
@@ -67,10 +96,27 @@ UNCONDITIONAL_JUMP_INSTRUCTIONS: Dict[int, Set[int]] = {
         RISCV_INS_C_JR,
     },
     CS_ARCH_PPC: {PPC_INS_B, PPC_INS_BA, PPC_INS_BL, PPC_INS_BLA},
+    CS_ARCH_SYSTEMZ: {
+        SYSTEMZ_INS_J,
+        SYSTEMZ_INS_JL,
+        SYSTEMZ_INS_B,
+        SYSTEMZ_INS_BR,
+        SYSTEMZ_INS_BAL,
+        SYSTEMZ_INS_BALR,
+        SYSTEMZ_INS_BRAS,
+        SYSTEMZ_INS_BRASL,
+    },
+    CS_ARCH_LOONGARCH: {
+        LOONGARCH_INS_B,
+        LOONGARCH_INS_BL,
+        LOONGARCH_INS_JIRL,
+        LOONGARCH_INS_ALIAS_JR,
+        LOONGARCH_INS_CALL36,
+    },
 }
 
 # See: https://github.com/capstone-engine/capstone/issues/2448
-BRANCH_AND_LINK_INSTRUCTIONS: Dict[int, Set[int]] = defaultdict(set)
+BRANCH_AND_LINK_INSTRUCTIONS: dict[int, set[int]] = defaultdict(set)
 BRANCH_AND_LINK_INSTRUCTIONS[CS_ARCH_MIPS] = {
     MIPS_INS_BAL,
     MIPS_INS_BLTZAL,
@@ -78,12 +124,11 @@ BRANCH_AND_LINK_INSTRUCTIONS[CS_ARCH_MIPS] = {
     MIPS_INS_JALR,
 }
 
-# Everything that is a CALL or a RET is a unconditional jump
-GENERIC_UNCONDITIONAL_JUMP_GROUPS = {CS_GRP_CALL, CS_GRP_RET, CS_GRP_IRET}
 # All branch-like instructions - jumps thats are non-call and non-ret - should have one of these two groups in Capstone
 GENERIC_JUMP_GROUPS = {CS_GRP_JUMP, CS_GRP_BRANCH_RELATIVE}
+
 # All Capstone jumps should have at least one of these groups
-ALL_JUMP_GROUPS = GENERIC_JUMP_GROUPS | GENERIC_UNCONDITIONAL_JUMP_GROUPS
+ALL_JUMP_GROUPS = GENERIC_JUMP_GROUPS | {CS_GRP_CALL, CS_GRP_RET, CS_GRP_IRET}
 
 # All non-ret jumps
 FORWARD_JUMP_GROUP = {CS_GRP_CALL} | GENERIC_JUMP_GROUPS
@@ -111,12 +156,14 @@ class SplitType(Enum):
 # Only use within the instruction.__repr__ to give a nice output
 CAPSTONE_ARCH_MAPPING_STRING = {
     CS_ARCH_ARM: "arm",
-    CS_ARCH_ARM64: "aarch64",
+    CS_ARCH_AARCH64: "aarch64",
     CS_ARCH_X86: "x86",
     CS_ARCH_PPC: "powerpc",
     CS_ARCH_MIPS: "mips",
     CS_ARCH_SPARC: "sparc",
     CS_ARCH_RISCV: "RISCV",
+    CS_ARCH_SYSTEMZ: "s390x",
+    CS_ARCH_LOONGARCH: "loongarch",
 }
 
 
@@ -127,16 +174,15 @@ class PwndbgInstruction(Protocol):
     size: int
     mnemonic: str
     op_str: str
-    groups: Set[int]
+    groups: set[int]
     id: int
-    operands: List[EnhancedOperand]
+    operands: list[EnhancedOperand]
     asm_string: str
     next: int
     target: int
     target_string: str | None
     target_const: bool | None
     condition: InstructionCondition
-    declare_conditional: bool | None
     declare_is_unconditional_jump: bool
     force_unconditional_jump_target: bool
     annotation: str | None
@@ -146,6 +192,7 @@ class PwndbgInstruction(Protocol):
     causes_branch_delay: bool
     split: SplitType
     emulated: bool
+    register_writes: dict[int, int]
 
     @property
     def call_like(self) -> bool: ...
@@ -165,6 +212,9 @@ class PwndbgInstruction(Protocol):
     @property
     def is_conditional_jump_taken(self) -> bool: ...
 
+    @property
+    def jump_result_is_known(self) -> bool: ...
+
     @property
     def bytes(self) -> bytearray: ...
 
@@ -178,7 +228,7 @@ class PwndbgInstruction(Protocol):
 # Pwndbg commands like "nextcall" that need to know the instructions target to set breakpoints
 # The information in this class is backed by metadata from Capstone
 class PwndbgInstructionImpl(PwndbgInstruction):
-    def __init__(self, cs_insn: CsInsn) -> None:
+    def __init__(self, cs_insn: CsInsn, padding: int = 6) -> None:
         self.cs_insn: CsInsn = cs_insn
         """
         The underlying Capstone instruction object.
@@ -202,32 +252,38 @@ class PwndbgInstructionImpl(PwndbgInstruction):
         Ex: 'RAX, RDX'
         """
 
-        self.groups: Set[int] = set(cs_insn.groups)
+        self.groups: set[int] = set(cs_insn.groups)
         """
         Capstone instruction groups that we belong to.
         Groups that apply to all architectures: CS_GRP_INVALID | CS_GRP_JUMP | CS_GRP_CALL | CS_GRP_RET | CS_GRP_INT | CS_GRP_IRET | CS_GRP_PRIVILEGE | CS_GRP_BRANCH_RELATIVE
         """
 
-        self.id: int = cs_insn.id
+        self.id: int = cs_insn.alias_id if cs_insn.is_alias else cs_insn.id
         """
         The underlying Capstone ID for the instruction
+        If it's an alias, use the id of the alias
+
         Examples: X86_INS_JMP, X86_INS_CALL, RISCV_INS_C_JAL
         """
 
         # For ease, for x86 we will assume Intel syntax (destination operand first).
         # However, Capstone will disassemble using the `set disassembly-flavor` preference,
         # and the order of operands are read left to right into the .operands array. So we flip operand order if AT&T
-        if self.cs_insn._cs.syntax == CS_OPT_SYNTAX_ATT:
+        if self.cs_insn._cs.arch == CS_ARCH_X86 and self.cs_insn._cs.syntax == CS_OPT_SYNTAX_ATT:
             self.cs_insn.operands.reverse()
 
-        self.operands: List[EnhancedOperand] = [EnhancedOperand(op) for op in self.cs_insn.operands]
+        self.operands: list[EnhancedOperand] = [EnhancedOperand(op) for op in self.cs_insn.operands]
 
         # ***********
         # The following member variables are set during instruction enhancement
         # in pwndbg.aglib.disasm.arch.py
         # ***********
 
-        self.asm_string: str = f"{self.mnemonic:<6} {self.op_str}"
+        self.asm_string: str = (
+            f"{self.mnemonic:<{padding}} {self.op_str}"
+            if self.op_str
+            else f"{self.mnemonic:<{padding}}"
+        )
         """
         The full string representing the instruction - `mov    rdi, rsp` with appropriate padding.
 
@@ -283,27 +339,12 @@ class PwndbgInstructionImpl(PwndbgInstruction):
         FALSE if the instruction has a conditional action, and we know it is not taken.
         """
 
-        self.declare_conditional: bool | None = None
-        """
-        This field is used to declare if the instruction is a conditional instruction.
-        In most cases, we can determine this purely based on the instruction ID, and this field is irrelevent.
-        However, in some arches, like Arm, the same instruction can be made conditional by certain instruction attributes.
-        Ex:
-            Arm, `bls` instruction. This is encoded as a `b` (Capstone ID 11) under the code, with an additional condition code field.
-            In this case, sometimes a `b` instruction (ID 11) is unconditional (always branches), in other cases it is conditional.
-            We use this field to disambiguate these cases.
-
-        True if we manually determine this instruction is a conditional instruction
-        False if it's not a conditional instruction
-        None if we don't have a determination (most cases)
-        """
-
         self.declare_is_unconditional_jump: bool = False
         """
         This field is used to declare that this instruction is an unconditional jump.
-        Most of the type, we depend on Capstone groups to check for jump instructions,
-        but sometimes these are lacking, such as in the case of general-purpose instructions
-        where the PC is the destination register, such as Arm `add`, `sub`, `ldr`, and `pop` instructions.
+        Most of the time, we depend on Capstone groups to check for jump instructions.
+        However, some instructions become branches depending on the operands,
+        such as Arm `add`, `sub`, `ldr`, `pop`, where PC is the destination register
 
         In these cases, we want to forcefully state that this instruction mutates the PC, so we set this attribute to True.
 
@@ -365,6 +406,12 @@ class PwndbgInstructionImpl(PwndbgInstruction):
         If the enhancement successfully used emulation for this instruction
         """
 
+        self.register_writes = {}
+        """
+        Mapping of Capstone register id to integer value. During enhancement, we might manually determine
+        that an instruction writes some value to a register, and this is stored here.
+        """
+
     @property
     def call_like(self) -> bool:
         """
@@ -412,7 +459,7 @@ class PwndbgInstructionImpl(PwndbgInstruction):
         This does not imply that we have resolved the .target
         """
         return (
-            self.declare_conditional is not False
+            self.declare_is_unconditional_jump is False
             and bool(self.groups & GENERIC_JUMP_GROUPS)
             and self.id not in UNCONDITIONAL_JUMP_INSTRUCTIONS[self.cs_insn._cs.arch]
         )
@@ -429,10 +476,8 @@ class PwndbgInstructionImpl(PwndbgInstruction):
         This does not imply that we have resolved the .target
         """
         return (
-            bool(self.groups & GENERIC_UNCONDITIONAL_JUMP_GROUPS)
+            self.declare_is_unconditional_jump
             or self.id in UNCONDITIONAL_JUMP_INSTRUCTIONS[self.cs_insn._cs.arch]
-            or self.declare_is_unconditional_jump
-            or self.declare_conditional is False
         )
 
     @property
@@ -452,6 +497,19 @@ class PwndbgInstructionImpl(PwndbgInstruction):
             )
         )
 
+    @property
+    def jump_result_is_known(self) -> bool:
+        """
+        True under the following conditions:
+        - If it's an unconditional jump, we know the target of the jump
+        - If it's a conditional jump, we know the target of the branch and know whether or not we take it
+        Otherwise, false
+        """
+        return self.has_jump_target and (
+            (self.is_unconditional_jump)
+            or (self.is_conditional_jump and self.condition != InstructionCondition.UNDETERMINED)
+        )
+
     @property
     def bytes(self) -> bytearray:
         """
@@ -476,9 +534,16 @@ class PwndbgInstructionImpl(PwndbgInstruction):
     def __repr__(self) -> str:
         operands_str = " ".join([repr(op) for op in self.operands])
 
-        info = f"""{self.mnemonic} {self.op_str} at {self.address:#x} (size={self.size}) (arch: {CAPSTONE_ARCH_MAPPING_STRING.get(self.cs_insn._cs.arch,None)})
+        hex_register_writes = {
+            self.cs_insn.reg_name(reg_id): hex(reg_value)
+            for reg_id, reg_value in self.register_writes.items()
+        }
+
+        info = f"""{self.mnemonic} {self.op_str} at {self.address:#x} (size={self.size}) (arch: {CAPSTONE_ARCH_MAPPING_STRING.get(self.cs_insn._cs.arch, None)})
+        Bytes: {pwnlib.util.fiddling.enhex(self.bytes)}
         ID: {self.id}, {self.cs_insn.insn_name()}
-        Raw asm: {'%-06s %s' % (self.mnemonic, self.op_str)}
+        Capstone ID/Alias ID: {self.cs_insn.id} / {self.cs_insn.alias_id if self.cs_insn.is_alias else "None"}
+        Raw asm: f"{self.mnemonic:-<6} {self.op_str}"
         New asm: {self.asm_string}
         Next: {self.next:#x}
         Target: {hex(self.target) if self.target is not None else None}, Target string={self.target_string or ""}, const={self.target_const}
@@ -488,18 +553,32 @@ class PwndbgInstructionImpl(PwndbgInstruction):
         Operands: [{operands_str}]
         Conditional jump: {self.is_conditional_jump}. Taken: {self.is_conditional_jump_taken}
         Unconditional jump: {self.is_unconditional_jump}
-        Declare unconditional: {self.declare_conditional}
         Declare unconditional jump: {self.declare_is_unconditional_jump}
         Force jump target: {self.force_unconditional_jump_target}
         Can change PC: {self.has_jump_target}
         Syscall: {self.syscall if self.syscall is not None else ""} {self.syscall_name if self.syscall_name is not None else "N/A"}
         Causes Delay slot: {self.causes_branch_delay}
         Split: {SplitType(self.split).name}
-        Call-like: {self.call_like}"""
+        Call-like: {self.call_like}
+        Register writes: {hex_register_writes}"""
+
+        try:
+            regs_read, regs_written = self.cs_insn.regs_access()
+            info += f"\n\tCapstone regs read: {[self.cs_insn.reg_name(reg) for reg in regs_read]}"
+            info += (
+                f"\n\tCapstone regs written: {[self.cs_insn.reg_name(reg) for reg in regs_written]}"
+            )
+        except CsError:
+            # Not all architectures support the .reg_access() API
+            pass
 
         # Hacky, but this is just for debugging
         if hasattr(self.cs_insn, "cc"):
             info += f"\n\tARM condition code: {self.cs_insn.cc}"
+            info += f"\n\tThumb mode: {1 if self.cs_insn._cs._mode & CS_MODE_THUMB else 0}"
+
+        if hasattr(self.cs_insn, "cc_field"):
+            info += f"\n\tSPARC cc_field: {self.cs_insn.cc_field}"
 
         return info
 
@@ -612,7 +691,7 @@ class EnhancedOperand:
 # Represents a disassembled instruction
 # Conforms to the PwndbgInstruction interface
 class ManualPwndbgInstruction(PwndbgInstruction):
-    def __init__(self, address: int) -> None:
+    def __init__(self, address: int, padding: int = 6) -> None:
         """
         This class provides an implementation of PwndbgInstruction for cases where the architecture
         at hand is not supported by the Capstone disassembler. The backing information is sourced from
@@ -640,7 +719,11 @@ class ManualPwndbgInstruction(PwndbgInstruction):
 
         self.operands = []
 
-        self.asm_string = f"{self.mnemonic:<6} {self.op_str}"
+        self.asm_string = (
+            f"{self.mnemonic:<{padding}} {self.op_str}"
+            if self.op_str
+            else f"{self.mnemonic:<{padding}}"
+        )
 
         self.next = address + self.size
         self.target = self.next
@@ -649,7 +732,6 @@ class ManualPwndbgInstruction(PwndbgInstruction):
 
         self.condition = InstructionCondition.UNDETERMINED
 
-        self.declare_conditional = None
         self.declare_is_unconditional_jump = False
         self.force_unconditional_jump_target = False
 
@@ -666,6 +748,8 @@ class ManualPwndbgInstruction(PwndbgInstruction):
 
         self.emulated = False
 
+        self.register_writes = {}
+
     @property
     def bytes(self) -> bytearray:
         # GDB simply doesn't provide us with the raw bytes.
@@ -697,6 +781,10 @@ class ManualPwndbgInstruction(PwndbgInstruction):
     def is_conditional_jump_taken(self) -> bool:
         return False
 
+    @property
+    def jump_result_is_known(self) -> bool:
+        return False
+
     @override
     def op_find(self, op_type: int, position: int) -> EnhancedOperand:
         # raise NotImplementedError, because if this is called it indicates a bug elsewhere in the codebase.
