@@ -10,12 +10,12 @@ import errno
 import os
 import shutil
 import tempfile
-from typing import Iterator
-from typing import Tuple
+from collections.abc import Iterator
 
 import pwndbg.aglib.proc
 import pwndbg.aglib.qemu
 import pwndbg.aglib.remote
+import pwndbg.dbg_mod
 import pwndbg.lib.cache
 
 _remote_files_dir = None
@@ -42,14 +42,14 @@ def get_proc_exe_file() -> str:
     """
     Returns the local path to the debugged file name.
     """
-    return get_file(pwndbg.aglib.proc.exe, try_local_path=True)
+    return get_file(pwndbg.aglib.proc.exe(), try_local_path=True)
 
 
 @pwndbg.lib.cache.cache_until("start")
 def can_download_remote_file() -> bool:
     if not pwndbg.aglib.remote.is_remote():
         return False
-    elif pwndbg.aglib.qemu.is_qemu_kernel():
+    if pwndbg.aglib.qemu.is_qemu_kernel():
         return False
 
     # Some[1] gdb servers don't implement vFile packets.
@@ -82,15 +82,11 @@ def get_file(path: str, try_local_path: bool = False) -> str:
         The local path to the file
     """
     has_target_prefix = path.startswith("target:")
-    has_good_prefix = path.startswith(("/", "./", "../")) or has_target_prefix
-    if not has_good_prefix:
-        raise OSError("get_file called with incorrect path", errno.ENOENT)
-
     if has_target_prefix:
         path = path[7:]  # len('target:') == 7
 
     local_path = path
-    if not pwndbg.aglib.remote.is_remote():
+    if not pwndbg.aglib.remote.is_remote() or pwndbg.aglib.qemu.is_qemu_kernel():
         if not os.path.exists(local_path):
             raise OSError(f"File '{local_path}' does not exist", errno.ENOENT)
 
@@ -160,6 +156,13 @@ def is_vfile_qemu_user_bug() -> bool:
     if not pwndbg.aglib.qemu.is_qemu_usermode():
         return False
 
+    # The bug was fixed in QEMU version v10.0.0-rc0 but the qGDBServerVersion packet wasn't added until v10.1.0-rc0
+    # Therefore, check the simple case of whether the QEMU version >= v10.1.0
+    # and fallback to the old check method otherwise to handle the gap between v10.0.0 and v10.1.0
+    version = pwndbg.aglib.qemu.qemu_gdbserver_version()
+    if version is not None and version >= (10, 1, 0):
+        return False
+
     # On a bugged QEMU version, the response is `F-1,36`
     # On a fixed QEMU version, the response is `F-1,24`
     # This performs the syscall: `openat(0, "/\01*256", O_RDONLY|0x20) = -1 ENAMETOOLONG (File name too long)`
@@ -268,7 +271,7 @@ def gdb_memtox_inverse(data: bytes) -> bytes:
     return buffer
 
 
-def vfile_pread(fd: int, size: int, offset: int) -> Tuple[int, bytes]:
+def vfile_pread(fd: int, size: int, offset: int) -> tuple[int, bytes]:
     """
     Reads data from a file descriptor.
 
