@@ -7,25 +7,22 @@ import struct
 import textwrap
 from abc import ABC
 from abc import abstractmethod
+from collections.abc import Iterable
 from dataclasses import dataclass
 from enum import IntEnum
 from typing import Any
-from typing import Dict
-from typing import Iterable
-from typing import List
 from typing import Literal
-from typing import Set
-from typing import Tuple
 from typing import cast
 
 import pwndbg
-import pwndbg.aglib.arch
+import pwndbg.aglib
 import pwndbg.aglib.elf
 import pwndbg.aglib.file
 import pwndbg.aglib.memory
 import pwndbg.aglib.proc
 import pwndbg.aglib.symbol
 import pwndbg.color.memory
+import pwndbg.dintegration
 import pwndbg.hexdump
 import pwndbg.lib.cache
 from pwndbg.color import generateColorFunction
@@ -40,7 +37,9 @@ indent_amount = pwndbg.config.add_param(
 )
 
 debug_color = theme.add_color_param(
-    "go-dump-debug", "blue", "color for 'go-dump' command's debug info when --debug is specified"
+    "go-dump-debug",
+    "blue",
+    "color for 'go-dump' command's debug info when --debug is specified",
 )
 
 
@@ -49,11 +48,49 @@ def word_size() -> int:
     """
     Gets the Go word size for the current architecture.
 
-    Values taken from https://github.com/golang/go/blob/20b79fd5775c39061d949569743912ad5e58b0e7/src/go/types/sizes.go#L233-L252
+    Values taken from https://github.com/golang/go/blob/49cdf0c42e320dfed044baa551610f081eafb781/src/cmd/compile/internal/types2/sizes.go#L230-L249
     """
-    return {"i386": 4, "x86-64": 8, "aarch64": 8, "arm": 4, "rv64": 8, "powerpc": 8, "sparc": 8}[
-        pwndbg.aglib.arch.name
-    ]
+    return {
+        "x86-64": 8,
+        "i386": 4,
+        # Go cannot target i8086
+        "arm": 4,
+        # Go cannot target armcm
+        "aarch64": 8,
+        "powerpc": 8,
+        "sparc": 8,
+        # Go cannot target rv32 (but gccgo can)
+        "rv64": 8,
+        "mips": pwndbg.aglib.arch.ptrsize,  # pwndbg uses mips for both 32 and 64 bit
+        "loongarch64": 8,
+        "s390x": 8,
+    }[pwndbg.aglib.arch.name]
+
+
+@pwndbg.lib.cache.cache_until("start", "stop", "objfile")
+def max_align() -> int:
+    """
+    Gets the Go maximum alignment for the current architecture.
+
+    Values taken from https://github.com/golang/go/blob/49cdf0c42e320dfed044baa551610f081eafb781/src/cmd/compile/internal/types2/sizes.go#L230-L249
+    """
+    # Note: gccgo has max alignment 8 for arm and mips, which is different from the default compiler
+    # Currently, no attempt is made to support gccgo, but it may be worth pursuing in the future
+    return {
+        "x86-64": 8,
+        "i386": 4,
+        # Go cannot target i8086
+        "arm": 4,
+        # Go cannot target armcm
+        "aarch64": 8,
+        "powerpc": 8,
+        "sparc": 8,
+        # Go cannot target rv32 (but gccgo can)
+        "rv64": 8,
+        "mips": pwndbg.aglib.arch.ptrsize,  # pwndbg uses mips for both 32 and 64 bit
+        "loongarch64": 8,
+        "s390x": 8,
+    }[pwndbg.aglib.arch.name]
 
 
 def _align(offset: int, n: int) -> int:
@@ -61,7 +98,7 @@ def _align(offset: int, n: int) -> int:
     return ret - ret % n
 
 
-def compute_offsets(fields: Iterable[Tuple[int, int]]) -> List[int]:
+def compute_offsets(fields: Iterable[tuple[int, int]]) -> list[int]:
     """
     Given a list of (size, alignment) for struct field types,
     returns a list of field offsets for the struct.
@@ -82,7 +119,7 @@ def compute_offsets(fields: Iterable[Tuple[int, int]]) -> List[int]:
     return ret
 
 
-def compute_named_offsets(fields: Iterable[Tuple[str, int, int]]) -> Dict[str, int]:
+def compute_named_offsets(fields: Iterable[tuple[str, int, int]]) -> dict[str, int]:
     """
     Like compute_offsets, but takes in field names and returns a dictionary
     mapping field name to offset instead.
@@ -91,6 +128,7 @@ def compute_named_offsets(fields: Iterable[Tuple[str, int, int]]) -> Dict[str, i
     """
     offsets = compute_offsets([f[1:] for f in fields])
     ret = dict(zip([f[0] for f in fields] + ["$size"], offsets))
+    ret["$align"] = max(f[2] for f in fields)
     return ret
 
 
@@ -108,14 +146,12 @@ class FormatOpts:
     def fmt_int(self, val: int) -> str:
         if self.int_hex:
             return hex(val)
-        else:
-            return str(val)
+        return str(val)
 
     def fmt_float(self, val: float) -> str:
         if self.float_decimals is not None:
             return format(val, f".{self.float_decimals}f")
-        else:
-            return str(val)
+        return str(val)
 
     def fmt_str(self, val: str) -> str:
         return json.dumps(val)
@@ -129,8 +165,7 @@ class FormatOpts:
     def fmt_debug(self, val: str, default: str = "") -> str:
         if self.debug:
             return generateColorFunction(debug_color)(val)
-        else:
-            return default
+        return default
 
     def fmt_elems(self, elems: Iterable[str]) -> str:
         if not self.pretty:
@@ -144,7 +179,9 @@ class FormatOpts:
         return f"\n{_indent(joined)}\n"
 
     def fmt_ptr(self, val: int) -> str:
-        return pwndbg.color.memory.get_address_and_symbol(val)
+        return pwndbg.color.memory.get_address_and_symbol(
+            val, pwndbg.dintegration.manager.get_stack_var_dict_all()
+        )
 
 
 @dataclass
@@ -165,6 +202,15 @@ class Type(ABC):
         """
         pass
 
+    @abstractmethod
+    def align(self) -> int:
+        """
+        Returns the alignment of a type in bytes.
+
+        Used for computing array and struct layouts.
+        """
+        pass
+
     @abstractmethod
     def get_typename(self) -> str:
         """
@@ -181,7 +227,7 @@ class Type(ABC):
 
         return _cyclic_helper(self, set())
 
-    def additional_metadata(self) -> List[str]:
+    def additional_metadata(self) -> list[str]:
         """
         Returns a list of lines of additional metadata to dump from the `go-type` command.
         """
@@ -191,17 +237,16 @@ class Type(ABC):
         return self.get_typename()
 
 
-def _cyclic_helper(val: Any, seen: Set[int]) -> bool:
+def _cyclic_helper(val: Any, seen: set[int]) -> bool:
     if isinstance(val, Type):
         k = id(val)
         if k in seen:
             return True
         seen.add(k)
         return any(_cyclic_helper(v, seen) for v in val.__dict__.values())
-    elif isinstance(val, (list, tuple)):
+    if isinstance(val, (list, tuple)):
         return any(_cyclic_helper(v, seen) for v in val)
-    else:
-        return False
+    return False
 
 
 def load_uint(data: bytes, endian: Literal["little", "big"] | None = None) -> int:
@@ -235,7 +280,7 @@ def emit_warning(msg: str):
 def get_elf() -> pwndbg.aglib.elf.ELFInfo | None:
     try:
         return pwndbg.aglib.elf.get_elf_info_rebased(
-            pwndbg.aglib.file.get_proc_exe_file(), pwndbg.aglib.proc.binary_base_addr
+            pwndbg.aglib.file.get_proc_exe_file(), pwndbg.aglib.proc.binary_base_addr()
         )
     except OSError:
         return None
@@ -252,7 +297,7 @@ def read_buildversion(addr: int) -> str:
 
 
 @pwndbg.lib.cache.cache_until("objfile")
-def get_go_version() -> Tuple[int, ...] | None:
+def get_go_version() -> tuple[int, ...] | None:
     """
     Try to determine the Go version used to compile the binary.
 
@@ -269,7 +314,8 @@ def get_go_version() -> Tuple[int, ...] | None:
         if elf is None:
             return None
         buildinfo = next(
-            (cast(int, s["sh_addr"]) for s in elf.sections if s["x_name"] == ".go.buildinfo"), None
+            (cast(int, s["sh_addr"]) for s in elf.sections if s["x_name"] == ".go.buildinfo"),
+            None,
         )
         # again, could do linear search
         if buildinfo is None:
@@ -285,11 +331,14 @@ def get_go_version() -> Tuple[int, ...] | None:
     if not version_string.startswith("go"):
         emit_warning(f"Go version string {version_string!r} doesn't start with 'go'")
         return None
+
+    # Cleanup a string that looks like "go1.25.5 X:nodwarf5"
+    version_string = version_string.split(" ")[0]
     return tuple(int(x) for x in version_string[2:].split("."))
 
 
 @pwndbg.lib.cache.cache_until("objfile")
-def _get_moduledata_types() -> Tuple[Tuple[int, int], ...] | None:
+def _get_moduledata_types() -> tuple[tuple[int, int], ...] | None:
     ret = []
     try:
         md = pwndbg.aglib.symbol.lookup_symbol("runtime.firstmoduledata")
@@ -324,7 +373,8 @@ def _guess_moduledata_types() -> int | None:
     elf = get_elf()
     if elf is not None:
         addr = next(
-            (cast(int, x["sh_addr"]) for x in elf.sections if x["x_name"] == ".rodata"), None
+            (cast(int, x["sh_addr"]) for x in elf.sections if x["x_name"] == ".rodata"),
+            None,
         )
         return addr
     return None
@@ -459,14 +509,18 @@ class BackrefType(Type):
             f"Cannot get size of placeholder type {type(self).__name__}. Perhaps the type is ill-formed? (e.g. struct that contains itself without indirection)"
         )
 
+    def align(self) -> int:
+        raise NotImplementedError(
+            f"Cannot get alignment of placeholder type {type(self).__name__}. Perhaps the type is ill-formed? (e.g. struct that contains itself without indirection)"
+        )
+
     def get_typename(self) -> str:
         if self.meta:
             return f"runtime({self.meta.size}){self.meta.addr:#x}"
-        else:
-            return "..."
+        return "..."
 
 
-def decode_runtime_type(addr: int, keep_backrefs: bool = False) -> Tuple[GoTypeMeta, Type | None]:
+def decode_runtime_type(addr: int, keep_backrefs: bool = False) -> tuple[GoTypeMeta, Type | None]:
     """
     Decodes a runtime reflection type from memory, returning a (meta, type) tuplee.
 
@@ -487,7 +541,7 @@ def decode_runtime_type(addr: int, keep_backrefs: bool = False) -> Tuple[GoTypeM
     }
     """
 
-    cache: Dict[int, Tuple[GoTypeMeta, Type | None]] = {}
+    cache: dict[int, tuple[GoTypeMeta, Type | None]] = {}
     (meta, rec_ty) = _inner_decode_runtime_type(addr, cache)
 
     if not keep_backrefs:
@@ -495,7 +549,7 @@ def decode_runtime_type(addr: int, keep_backrefs: bool = False) -> Tuple[GoTypeM
     return (meta, rec_ty)
 
 
-def _remove_backrefs(ty: Any, cache: Dict[int, Tuple[GoTypeMeta, Type | None]]) -> Any:
+def _remove_backrefs(ty: Any, cache: dict[int, tuple[GoTypeMeta, Type | None]]) -> Any:
     """
     Helper function to replace all _BackrefType instances after the cache is fully resolved.
 
@@ -503,21 +557,20 @@ def _remove_backrefs(ty: Any, cache: Dict[int, Tuple[GoTypeMeta, Type | None]])
     """
     if isinstance(ty, BackrefType):
         return cache[ty.key][1]
-    elif isinstance(ty, Type):
+    if isinstance(ty, Type):
         d = ty.__dict__
         for k, v in d.items():
             d[k] = _remove_backrefs(v, cache)
         return ty
-    elif isinstance(ty, (list, tuple)):
+    if isinstance(ty, (list, tuple)):
         constructor = type(ty)
         return constructor(_remove_backrefs(x, cache) for x in ty)
-    else:
-        return ty
+    return ty
 
 
 def _inner_decode_runtime_type(
-    addr: int, cache: Dict[int, Tuple[GoTypeMeta, Type | None]]
-) -> Tuple[GoTypeMeta, Type | None]:
+    addr: int, cache: dict[int, tuple[GoTypeMeta, Type | None]]
+) -> tuple[GoTypeMeta, Type | None]:
     """
     Internal function for decode_runtime_type with a cache to avoid recursive types.
     """
@@ -566,15 +619,20 @@ def _inner_decode_runtime_type(
     size = load(offsets["Size_"], word)
     align = load(offsets["Align_"], 1)
     meta = GoTypeMeta(
-        name, kind, addr, size=size, align=align, direct_iface=(kind_raw & (1 << 5)) != 0
+        name,
+        kind,
+        addr,
+        size=size,
+        align=align,
+        direct_iface=(kind_raw & (1 << 5)) != 0,
     )
     cache[addr] = (meta, BackrefType(meta, addr))
     simple_name = kind.get_simple_name()
 
-    def compute() -> Tuple[GoTypeMeta, Type | None]:
+    def compute() -> tuple[GoTypeMeta, Type | None]:
         if simple_name is not None:
             return (meta, BasicType(meta, simple_name))
-        elif kind == GoTypeKind.FUNC:
+        if kind == GoTypeKind.FUNC:
             in_count = load(offsets["$size"], 2)
             out_count = load(offsets["$size"] + 2, 2)
             vararg_bit = 1 << 15
@@ -602,42 +660,47 @@ def _inner_decode_runtime_type(
                     info.append(f"Argument {i}{suffix}:")
                 else:
                     info.append(f"Return value {i - in_count}:")
-                info += [f"    Type name: {ty_meta.name}", f"    Type addr: {ty_ptr:#x}"]
+                info += [
+                    f"    Type name: {ty_meta.name}",
+                    f"    Type addr: {ty_ptr:#x}",
+                ]
             return (meta, BasicType(meta, "funcptr", info))
-        elif kind == GoTypeKind.ARRAY:
+        if kind == GoTypeKind.ARRAY:
             elem_ty_ptr = load(offsets["$size"], word)
             arr_len = load(offsets["$size"] + word * 2, word)
             elem_meta, elem_ty = _inner_decode_runtime_type(elem_ty_ptr, cache)
             # reserialize name to fix inconsistencies
             meta.name = f"[{arr_len}]{elem_meta.name}"
             return (meta, elem_ty and ArrayType(meta, elem_ty, arr_len))
-        elif kind == GoTypeKind.INTERFACE:
+        if kind == GoTypeKind.INTERFACE:
             methods_count = load(offsets["$size"] + word * 2, word)
             if methods_count == 0:
                 return (meta, BasicType(meta, "any"))
-            elif type_start is None:
-                return (meta, BasicType(meta, "interface", [f"Method count: {methods_count}"]))
-            else:
-                info = []
-                methods_ptr = load(offsets["$size"] + word, word)
-                for i in range(methods_count):
-                    base = methods_ptr + i * 8
-                    meth_name_off = load_uint(pwndbg.aglib.memory.read(base, 4))
-                    inner_off = load_uint(pwndbg.aglib.memory.read(base + 4, 4))
-                    bmeth_name = read_type_name(type_start + meth_name_off)
-                    inner_ty_ptr = type_start + inner_off
-                    (inner_meta, _) = _inner_decode_runtime_type(inner_ty_ptr, cache)
-                    try:
-                        meth_name = bmeth_name.decode()
-                    except UnicodeDecodeError:
-                        meth_name = repr(bytes(bmeth_name))
-                    info += [
-                        f"Method {meth_name}:",
-                        f"    Type name: {inner_meta.name}",
-                        f"    Type addr: {inner_ty_ptr:#x}",
-                    ]
-                return (meta, BasicType(meta, "interface", info))
-        elif kind == GoTypeKind.MAP:
+            if type_start is None:
+                return (
+                    meta,
+                    BasicType(meta, "interface", [f"Method count: {methods_count}"]),
+                )
+            info = []
+            methods_ptr = load(offsets["$size"] + word, word)
+            for i in range(methods_count):
+                base = methods_ptr + i * 8
+                meth_name_off = load_uint(pwndbg.aglib.memory.read(base, 4))
+                inner_off = load_uint(pwndbg.aglib.memory.read(base + 4, 4))
+                bmeth_name = read_type_name(type_start + meth_name_off)
+                inner_ty_ptr = type_start + inner_off
+                (inner_meta, _) = _inner_decode_runtime_type(inner_ty_ptr, cache)
+                try:
+                    meth_name = bmeth_name.decode()
+                except UnicodeDecodeError:
+                    meth_name = repr(bytes(bmeth_name))
+                info += [
+                    f"Method {meth_name}:",
+                    f"    Type name: {inner_meta.name}",
+                    f"    Type addr: {inner_ty_ptr:#x}",
+                ]
+            return (meta, BasicType(meta, "interface", info))
+        if kind == GoTypeKind.MAP:
             key_ty_ptr = load(offsets["$size"], word)
             val_ty_ptr = load(offsets["$size"] + word, word)
             key_meta, key_ty = _inner_decode_runtime_type(key_ty_ptr, cache)
@@ -650,22 +713,22 @@ def _inner_decode_runtime_type(
             meta.name = f"map[{key_meta.name}]{val_meta.name}"
             # Go maps are actually pointers, but the map here is not
             return (meta, PointerType(meta, MapType(meta, key_ty, val_ty)))
-        elif kind == GoTypeKind.POINTER:
+        if kind == GoTypeKind.POINTER:
             elem_ty_ptr = load(offsets["$size"], word)
             elem_meta, elem_ty = _inner_decode_runtime_type(elem_ty_ptr, cache)
             # reserialize name to fix inconsistencies
             meta.name = f"*{elem_meta.name}"
             return (meta, elem_ty and PointerType(meta, elem_ty))
-        elif kind == GoTypeKind.SLICE:
+        if kind == GoTypeKind.SLICE:
             elem_ty_ptr = load(offsets["$size"], word)
             elem_meta, elem_ty = _inner_decode_runtime_type(elem_ty_ptr, cache)
             # reserialize name to fix inconsistencies
             meta.name = f"[]{elem_meta.name}"
             return (meta, elem_ty and SliceType(meta, elem_ty))
-        elif kind == GoTypeKind.STRUCT:
+        if kind == GoTypeKind.STRUCT:
             fields_ptr = load(offsets["$size"] + word, word)
             fields_count = load(offsets["$size"] + word * 2, word)
-            fields: List[Tuple[str, Type | str, int]] = []
+            fields: list[tuple[str, Type | str, int]] = []
             vers = get_go_version()
             if vers is not None and vers < (1, 19):
                 offset_shift = 1
@@ -687,14 +750,12 @@ def _inner_decode_runtime_type(
                     field_ty = field_meta.name
                 fields.append((field_name, field_ty, field_off))
             fields.sort(key=lambda f: f[2])
-            sz = load(offsets["Size_"], word)
             return (
                 meta,
-                StructType(meta, fields, sz, None if name.startswith("struct ") else name),
+                StructType(meta, fields, size, align, None if name.startswith("struct ") else name),
             )
-        else:
-            # currently channels and functions are unsupported
-            return (meta, None)
+        # currently channels and functions are unsupported
+        return (meta, None)
 
     ret = compute()
     if not isinstance(ret[1], BackrefType):
@@ -718,7 +779,8 @@ class BasicType(Type):
 
     name: str
     sz: int = dataclasses.field(init=False)
-    extra_meta: List[str] = dataclasses.field(default_factory=list)
+    algn: int = dataclasses.field(init=False)
+    extra_meta: list[str] = dataclasses.field(default_factory=list)
 
     def dump(self, addr: int, fmt: FormatOpts = FormatOpts()) -> str:
         val = pwndbg.aglib.memory.read(addr, self.size())
@@ -791,30 +853,41 @@ class BasicType(Type):
     def size(self) -> int:
         return self.sz
 
+    def align(self) -> int:
+        return min(self.algn, max_align())
+
     def get_typename(self) -> str:
         return self.name
 
-    def additional_metadata(self) -> List[str]:
+    def additional_metadata(self) -> list[str]:
         return self.extra_meta
 
     def __post_init__(self) -> None:
         ty = self.name
         if ty in ("int8", "uint8", "bool", "byte"):
             self.sz = 1
+            self.algn = 1
         elif ty in ("int16", "uint16"):
             self.sz = 2
+            self.algn = 2
         elif ty in ("int32", "uint32", "float32", "rune"):
             self.sz = 4
+            self.algn = 4
         elif ty in ("int64", "uint64", "float64", "complex64"):
             self.sz = 8
+            self.algn = 8
         elif ty == "complex128":
             self.sz = 16
+            self.algn = 8
         elif ty in ("int", "uint", "uintptr", "funcptr"):
             self.sz = word_size()
+            self.algn = word_size()
         elif ty == "string":
             self.sz = word_size() * 2
+            self.algn = word_size()
         elif ty in ("any", "interface"):
             self.sz = word_size() * 2
+            self.algn = word_size()
         else:
             raise ValueError(
                 f"Type {ty} is unknown. Use type hexdump[n] for an unknown type of size n."
@@ -849,10 +922,13 @@ class SliceType(Type):
     def size(self) -> int:
         return word_size() * 3
 
+    def align(self) -> int:
+        return word_size()
+
     def get_typename(self) -> str:
         return f"[]{self.inner}"
 
-    def additional_metadata(self) -> List[str]:
+    def additional_metadata(self) -> list[str]:
         if self.inner.meta:
             return [
                 f"Elem type name: {self.inner.meta.name}",
@@ -881,10 +957,13 @@ class PointerType(Type):
     def size(self) -> int:
         return word_size()
 
+    def align(self) -> int:
+        return min(word_size(), max_align())
+
     def get_typename(self) -> str:
         return f"*{self.inner}"
 
-    def additional_metadata(self) -> List[str]:
+    def additional_metadata(self) -> list[str]:
         # maps are returned as pointers to map in a parser
         # so show map metadata through the pointer metadata
         if isinstance(self.inner, MapType):
@@ -920,10 +999,13 @@ class ArrayType(Type):
     def size(self) -> int:
         return self.inner.size() * self.count
 
+    def align(self) -> int:
+        return self.inner.align()
+
     def get_typename(self) -> str:
         return f"[{self.count}]{self.inner}"
 
-    def additional_metadata(self) -> List[str]:
+    def additional_metadata(self) -> list[str]:
         if self.inner.meta:
             return [
                 f"        Length: {self.count}",
@@ -941,29 +1023,34 @@ class MapType(Type):
     Note that maps in Go are actually pointers to the inner map,
     but the map type printer here directly prints the inner map.
 
-    Maps don't have a simple layout, and may reasonably change,
-    but the last change was in 2017, so it probably won't.
-
-    The layout assumed is as follows (taken from src/runtime/map.go commit 1b4f1dc):
-
-    type hmap struct {
-        count      int
-        flags      uint8
-        B          uint8
-        noverflow  uint16
-        hash0      uint32
-        buckets    unsafe.Pointer
-        oldbuckets unsafe.Pointer
-        nevacuate  uintptr
-        extra      *mapextra
-    }
+    Maps don't have a simple layout, and may reasonably change.
     """
 
     key: Type
     val: Type
 
     @staticmethod
-    def field_offsets() -> Dict[str, int]:
+    def is_swiss() -> bool:
+        vers = get_go_version()
+        return vers is None or vers >= (1, 24)
+
+    @staticmethod
+    def field_offsets_noswiss() -> dict[str, int]:
+        """
+        The layout for pre-1.24 maps is as follows (taken from src/runtime/map.go commit 1b4f1dc):
+
+        type hmap struct {
+            count      int
+            flags      uint8
+            B          uint8
+            noverflow  uint16
+            hash0      uint32
+            buckets    unsafe.Pointer
+            oldbuckets unsafe.Pointer
+            nevacuate  uintptr
+            extra      *mapextra
+        }
+        """
         word = word_size()
         offsets = compute_named_offsets(
             [
@@ -980,10 +1067,91 @@ class MapType(Type):
         )
         return offsets
 
-    def dump(self, addr: int, fmt: FormatOpts = FormatOpts()) -> str:
+    @staticmethod
+    def field_offsets_swiss() -> dict[str, int]:
+        """
+        The layout for post-1.24 maps is as follows (taken from src/internal/runtime/map.go commit 4e63ae4):
+
+        type Map struct {
+            used              uint64
+            seed              uintptr
+            dirPtr            unsafe.Pointer
+            dirLen            int
+            globalDepth       uint8
+            globalShift       uint8
+            writing           uint8
+            tombstonePossible bool
+            clearSeq          uint64
+        }
+        """
+        word = word_size()
+        offsets = compute_named_offsets(
+            [
+                ("used", 8, 8),  # uint64
+                ("seed", word, word),  # uintptr
+                ("dirPtr", word, word),  # unsafe.Pointer
+                ("dirLen", word, word),  # int
+                ("globalDepth", 1, 1),  # uint8
+                ("globalShift", 1, 1),  # uint8
+                ("writing", 1, 1),  # uint8
+                ("tombstonePossible", 1, 1),  # bool
+                ("clearSeq", 8, 8),  # uint64
+            ]
+        )
+        return offsets
+
+    @staticmethod
+    def field_offsets_swiss_inner() -> dict[str, int]:
+        """
+        The layout for the inner swissmap is as follows (taken from src/internal/runtime/maps/table.go commit 4e63ae4):
+
+        type table struct {
+            used       uint16
+            capacity   uint16
+            growthLeft uint16
+            localDepth uint8
+            index      int
+            groups     groupsReference
+        }
+        """
+        word = word_size()
+        offsets = compute_named_offsets(
+            [
+                ("used", 2, 2),  # uint16
+                ("capacity", 2, 2),  # uint16
+                ("growthLeft", 2, 2),  # uint16
+                ("localDepth", 1, 1),  # uint8
+                ("index", word, word),  # uint8
+                ("groups", 16, 8),  # groupsReference
+            ]
+        )
+        return offsets
+
+    @classmethod
+    def field_offsets(cls) -> dict[str, int]:
+        if cls.is_swiss():
+            return cls.field_offsets_swiss()
+        return cls.field_offsets_noswiss()
+
+    @staticmethod
+    def format_entries(
+        entries: list[tuple[int, int, str, str]], fmt: FormatOpts = FormatOpts()
+    ) -> str:
+        # sort map by key, using integer comparison if possible
+        try:
+            entries.sort(key=lambda t: int(t[2], 0))
+        except ValueError:
+            entries.sort(key=lambda t: t[2])
+        formatted = []
+        for kp, vp, k, v in entries:
+            prefix = fmt.fmt_debug(f"(key @ {kp:#x}, val @ {vp:#x}) ")
+            formatted.append(f"{prefix}{k}: {v}")
+        return f"{{{fmt.fmt_elems(formatted)}}}"
+
+    def dump_noswiss(self, addr: int, fmt: FormatOpts = FormatOpts()) -> str:
         bucket_count = 8  # taken from src/internal/abi/map.go commit 1b4f1dc
         word = word_size()
-        offsets = self.field_offsets()
+        offsets = self.field_offsets_noswiss()
         val = pwndbg.aglib.memory.read(addr, offsets["$size"])
         load = lambda off, sz: load_uint(val[off : off + sz])
         num_buckets = 1 << load(offsets["B"], 1)
@@ -1020,24 +1188,64 @@ class MapType(Type):
                         v = self.val.dump(val_ptr, fmt)
                         ret.append((key_ptr, val_ptr, k, v))
                 bucket_ptr = load_uint(bucket[overflow_start : overflow_start + word])
-        # sort map by key, using integer comparison if possible
-        try:
-            ret.sort(key=lambda t: int(t[2], 0))
-        except ValueError:
-            ret.sort(key=lambda t: t[2])
-        formatted = []
-        for kp, vp, k, v in ret:
-            prefix = fmt.fmt_debug(f"(key @ {kp:#x}, val @ {vp:#x}) ")
-            formatted.append(f"{prefix}{k}: {v}")
-        return f"{{{fmt.fmt_elems(formatted)}}}"
+        return self.format_entries(ret, fmt)
+
+    def dump_swiss(self, addr: int, fmt: FormatOpts = FormatOpts()) -> str:
+        word = word_size()
+        offsets = self.field_offsets_swiss()
+        dir_ptr = load_uint(pwndbg.aglib.memory.read(addr + offsets["dirPtr"], word))
+        dir_len = load_uint(pwndbg.aglib.memory.read(addr + offsets["dirLen"], word))
+        slot_offsets = compute_named_offsets(
+            [
+                ("key", self.key.size(), self.key.align()),
+                ("elem", self.val.size(), self.val.align()),
+            ]
+        )
+        group_offsets = compute_named_offsets(
+            [("ctrls", 8, 8), ("slots", slot_offsets["$size"] * 8, slot_offsets["$align"])]
+        )
+        if dir_len == 0:
+            groups = [dir_ptr]
+        else:
+            table_ptrs = pwndbg.aglib.memory.read(dir_ptr, word * dir_len)
+            tables = [load_uint(table_ptrs[i * word : i * word + word]) for i in range(dir_len)]
+            table_offsets = self.field_offsets_swiss_inner()
+            groups = []
+            for table in tables:
+                groups_ref = pwndbg.aglib.memory.read(table + table_offsets["groups"], 16)
+                group_base = load_uint(groups_ref[:word])
+                group_count = load_uint(groups_ref[8:]) + 1
+                for _ in range(group_count):
+                    groups.append(group_base)
+                    group_base += group_offsets["$size"]
+        ret = []
+        for group_base in groups:
+            group = pwndbg.aglib.memory.read(group_base, group_offsets["$size"])
+            for i in range(8):
+                if group[i] & 0x80 == 0:
+                    off = 8 + slot_offsets["$size"] * i
+                    key_ptr = group_base + off + slot_offsets["key"]
+                    val_ptr = group_base + off + slot_offsets["elem"]
+                    k = self.key.dump(key_ptr, fmt)
+                    v = self.val.dump(val_ptr, fmt)
+                    ret.append((key_ptr, val_ptr, k, v))
+        return self.format_entries(ret, fmt)
+
+    def dump(self, addr: int, fmt: FormatOpts = FormatOpts()) -> str:
+        if self.is_swiss():
+            return self.dump_swiss(addr, fmt)
+        return self.dump_noswiss(addr, fmt)
 
     def size(self) -> int:
         return self.field_offsets()["$size"]
 
+    def align(self) -> int:
+        return min(8, max_align())
+
     def get_typename(self) -> str:
         return f"map[{self.key}]{self.val}"
 
-    def additional_metadata(self) -> List[str]:
+    def additional_metadata(self) -> list[str]:
         ret = []
         if self.key.meta:
             ret += [
@@ -1060,8 +1268,9 @@ class StructType(Type):
     and FIELDS is a semicolon-separated list of OFFSET:NAME:TYPE fields.
     """
 
-    fields: List[Tuple[str, Type | str, int]]
+    fields: list[tuple[str, Type | str, int]]
     sz: int
+    algn: int | None = None
     name: str | None = None
 
     def dump(self, addr: int, fmt: FormatOpts = FormatOpts()) -> str:
@@ -1079,17 +1288,28 @@ class StructType(Type):
     def size(self) -> int:
         return self.sz
 
+    def align(self) -> int:
+        if self.algn is None:
+            return max(
+                (ty.align() for (_, ty, _) in self.fields if isinstance(ty, Type)), default=1
+            )
+        return self.algn
+
     def get_typename(self) -> str:
         body = ";".join(
             f"{off}:{name}:{ty}" for (name, ty, off) in self.fields if not isinstance(ty, str)
         )
         return f"struct({self.sz}){{{body}}}"
 
-    def additional_metadata(self) -> List[str]:
+    def additional_metadata(self) -> list[str]:
         ret = []
         for name, ty, off in self.fields:
             if isinstance(ty, str) or not ty.meta:
-                ret += [f"Field {name}:", f"    Offset: {off} ({off:#x})", f"    Type: {ty}"]
+                ret += [
+                    f"Field {name}:",
+                    f"    Offset: {off} ({off:#x})",
+                    f"    Type: {ty}",
+                ]
             else:
                 ret += [
                     f"Field {name}:",
@@ -1112,17 +1332,23 @@ class RuntimeType(Type):
 
     sz: int
     addr: int
+    algn: int | None = dataclasses.field(init=False, default=None)
 
     def dump(self, addr: int, fmt: FormatOpts = FormatOpts()) -> str:
         (meta, ty) = decode_runtime_type(self.addr)
         if ty is not None:
             return f"({meta.name}) {ty.dump(addr, fmt)}"
-        else:
-            return f"[error resolving type `{meta.name}` at {addr:#x}]"
+        return f"[error resolving type `{meta.name}` at {addr:#x}]"
 
     def size(self) -> int:
         return self.sz
 
+    def align(self) -> int:
+        if self.algn is None:
+            (meta, _ty) = decode_runtime_type(self.addr)
+            self.algn = meta.align
+        return self.algn
+
     def get_typename(self) -> str:
         return f"runtime({self.sz}){self.addr:#x}"
 
@@ -1134,7 +1360,7 @@ _ident_rest = _ident_first | set(string.digits)
 hex_digits = set("0123456789abcdefABCDEFxX")
 
 
-def _parse_posint(ty: str) -> Tuple[int, str] | None:
+def _parse_posint(ty: str) -> tuple[int, str] | None:
     if not ty or ty[0] not in hex_digits:
         return None
     for i in range(1, len(ty)):
@@ -1148,7 +1374,7 @@ def _parse_posint(ty: str) -> Tuple[int, str] | None:
         return None
 
 
-def _parse_ident(ty: str) -> Tuple[str, str] | None:
+def _parse_ident(ty: str) -> tuple[str, str] | None:
     if not ty or ty[0] not in _ident_first:
         return None
     for i in range(1, len(ty)):
@@ -1159,7 +1385,7 @@ def _parse_ident(ty: str) -> Tuple[str, str] | None:
     return (ty[:i], ty[i:])
 
 
-def _parse_basic_ty(ty: str) -> Tuple[BasicType, str] | None:
+def _parse_basic_ty(ty: str) -> tuple[BasicType, str] | None:
     parse = _parse_ident(ty)
     if not parse:
         return None
@@ -1174,7 +1400,7 @@ def _parse_basic_ty(ty: str) -> Tuple[BasicType, str] | None:
         raise
 
 
-def _parse_slice_ty(ty: str) -> Tuple[SliceType, str] | None:
+def _parse_slice_ty(ty: str) -> tuple[SliceType, str] | None:
     if not ty.startswith("[]"):
         return None
     if (inner := _parse_type(ty[2:])) is None:
@@ -1182,7 +1408,7 @@ def _parse_slice_ty(ty: str) -> Tuple[SliceType, str] | None:
     return (SliceType(None, inner[0]), inner[1])
 
 
-def _parse_pointer_ty(ty: str) -> Tuple[PointerType, str] | None:
+def _parse_pointer_ty(ty: str) -> tuple[PointerType, str] | None:
     if not ty.startswith("*"):
         return None
     if (inner := _parse_type(ty[1:])) is None:
@@ -1190,7 +1416,7 @@ def _parse_pointer_ty(ty: str) -> Tuple[PointerType, str] | None:
     return (PointerType(None, inner[0]), inner[1])
 
 
-def _parse_array_ty(ty: str) -> Tuple[ArrayType, str] | None:
+def _parse_array_ty(ty: str) -> tuple[ArrayType, str] | None:
     if not ty.startswith("["):
         return None
     if (count := _parse_posint(ty[1:])) is None:
@@ -1202,7 +1428,7 @@ def _parse_array_ty(ty: str) -> Tuple[ArrayType, str] | None:
     return (ArrayType(None, inner[0], count[0]), inner[1])
 
 
-def _parse_map_ty(ty: str) -> Tuple[MapType, str] | None:
+def _parse_map_ty(ty: str) -> tuple[MapType, str] | None:
     if not ty.startswith("map["):
         return None
     if (key := _parse_type(ty[4:])) is None:
@@ -1214,7 +1440,7 @@ def _parse_map_ty(ty: str) -> Tuple[MapType, str] | None:
     return (MapType(None, key[0], val[0]), val[1])
 
 
-def _parse_struct_ty(ty: str) -> Tuple[StructType, str] | None:
+def _parse_struct_ty(ty: str) -> tuple[StructType, str] | None:
     if not ty.startswith("struct("):
         return None
     if (size_parse := _parse_posint(ty[7:])) is None:
@@ -1223,7 +1449,7 @@ def _parse_struct_ty(ty: str) -> Tuple[StructType, str] | None:
     if not cur.startswith("){"):
         return None
     cur = cur[2:]
-    fields: List[Tuple[str, Type | str, int]] = []
+    fields: list[tuple[str, Type | str, int]] = []
     is_first = True
     while cur:
         if cur.startswith("}"):
@@ -1250,7 +1476,7 @@ def _parse_struct_ty(ty: str) -> Tuple[StructType, str] | None:
     return None
 
 
-def _parse_runtime_ty(ty: str) -> Tuple[RuntimeType, str] | None:
+def _parse_runtime_ty(ty: str) -> tuple[RuntimeType, str] | None:
     if not ty.startswith("runtime("):
         return None
     if (size_parse := _parse_posint(ty[8:])) is None:
@@ -1264,7 +1490,7 @@ def _parse_runtime_ty(ty: str) -> Tuple[RuntimeType, str] | None:
     return (RuntimeType(None, size, addr), rest)
 
 
-def _parse_type(ty: str) -> Tuple[Type, str] | None:
+def _parse_type(ty: str) -> tuple[Type, str] | None:
     for f in [
         _parse_runtime_ty,
         _parse_struct_ty,
