@@ -8,53 +8,68 @@ from __future__ import annotations
 import re
 from itertools import chain
 
-import capstone
+from capstone6pwndbg import CS_GRP_INT
+from capstone6pwndbg import CS_GRP_RET
 
-import pwndbg.aglib.disasm
+import pwndbg.aglib
+import pwndbg.aglib.disasm.disassembly
 import pwndbg.aglib.proc
-import pwndbg.aglib.regs
+import pwndbg.aglib.vmmap
+import pwndbg.dbg_mod
+from pwndbg.aglib.disasm.instruction import PwndbgInstruction
 from pwndbg.color import message
-from pwndbg.dbg import BreakpointLocation
+from pwndbg.dbg_mod import BreakpointLocation
 
-interrupts = {capstone.CS_GRP_INT}
+interrupts = {CS_GRP_INT}
 
 
-def next_int(address=None):
+def next_int(address=None, honor_current_branch=False):
     """
     If there is a syscall in the current basic black,
     return the instruction of the one closest to $PC.
 
-    Otherwise, return None.
+    If honor_current_branch is True, then if the address is already a branch, return None.
+
+    If no interrupt exists or a jump is in the way, return None.
     """
     if address is None:
-        ins = pwndbg.aglib.disasm.one(pwndbg.aglib.regs.pc)
+        ins = pwndbg.aglib.disasm.disassembly.one(pwndbg.aglib.regs.pc)
         if not ins:
             return None
+        if honor_current_branch and ins.jump_like:
+            return None
         address = ins.next
 
-    ins = pwndbg.aglib.disasm.one(address)
+    ins = pwndbg.aglib.disasm.disassembly.one(address)
     while ins:
         if ins.jump_like:
             return None
-        elif ins.groups & interrupts:
+        if ins.groups & interrupts:
             return ins
-        ins = pwndbg.aglib.disasm.one(ins.next)
+        ins = pwndbg.aglib.disasm.disassembly.one(ins.next)
 
     return None
 
 
-def next_branch(address=None):
+def next_branch(address=None, including_current=False) -> PwndbgInstruction | None:
+    """
+    Return the next branch instruction that the process will encounter with repeated usage of the "nexti" command.
+
+    If including_current == True, then if the instruction at the address is already a branch, return it.
+    """
     if address is None:
-        ins = pwndbg.aglib.disasm.one(pwndbg.aglib.regs.pc)
+        ins = pwndbg.aglib.disasm.disassembly.one(pwndbg.aglib.regs.pc)
         if not ins:
             return None
+        if including_current and ins.jump_like:
+            return ins
         address = ins.next
 
-    ins = pwndbg.aglib.disasm.one(address)
+    ins = pwndbg.aglib.disasm.disassembly.one(address)
     while ins:
         if ins.jump_like:
             return ins
-        ins = pwndbg.aglib.disasm.one(ins.next)
+        ins = pwndbg.aglib.disasm.disassembly.one(ins.next)
 
     return None
 
@@ -67,7 +82,7 @@ def next_matching_until_branch(address=None, mnemonic=None, op_str=None):
     if address is None:
         address = pwndbg.aglib.regs.pc
 
-    ins = pwndbg.aglib.disasm.one(address)
+    ins = pwndbg.aglib.disasm.disassembly.one(address)
     while ins:
         # Check whether or not the mnemonic matches if it was specified
         mnemonic_match = ins.mnemonic.casefold() == mnemonic.casefold() if mnemonic else True
@@ -97,35 +112,53 @@ def next_matching_until_branch(address=None, mnemonic=None, op_str=None):
             # not trying to match the branch instruction itself.
             return None
 
-        ins = pwndbg.aglib.disasm.one(ins.next)
+        ins = pwndbg.aglib.disasm.disassembly.one(ins.next)
     return None
 
 
-async def break_next_branch(ec: pwndbg.dbg_mod.ExecutionController, address=None):
-    ins = next_branch(address)
+async def break_next_branch(
+    ec: pwndbg.dbg_mod.ExecutionController, address=None, including_current=False
+):
+    """
+    If including_current == True, do not step in case we are currently on a branch
+    """
+    ins = next_branch(address, including_current=including_current)
 
+    inf = pwndbg.dbg.selected_inferior()
+    if ins:
+        # If the branch we found was not at the current program counter, we should step to it.
+        # Otherwise, return the current instruction.
+        if ins.address != pwndbg.aglib.regs.pc:
+            with inf.break_at(BreakpointLocation(ins.address), internal=True) as bp:
+                await ec.cont(bp)
+        return ins
+
+
+async def break_next_interrupt(
+    ec: pwndbg.dbg_mod.ExecutionController, address=None, honor_current_branch=False
+) -> PwndbgInstruction | None:
+    """
+    Break at the next interrupt if there is one in the current basic block
+    and no jumps are between the current instruction and the interrupt.
+
+    If no such interrupt exists or a jump is in the way, return None.
+    """
+    ins = next_int(address, honor_current_branch=honor_current_branch)
     proc = pwndbg.dbg.selected_inferior()
     if ins:
         with proc.break_at(BreakpointLocation(ins.address), internal=True) as bp:
             await ec.cont(bp)
         return ins
 
-
-async def break_next_interrupt(ec: pwndbg.dbg_mod.ExecutionController, address=None):
-    ins = next_int(address)
-    proc = pwndbg.dbg.selected_inferior()
-    if ins:
-        with proc.break_at(BreakpointLocation(ins.address), internal=True) as bp:
-            await ec.cont(bp)
-        return ins
+    return None
 
 
 async def break_next_call(ec: pwndbg.dbg_mod.ExecutionController, symbol_regex=None):
     symbol_regex = re.compile(symbol_regex) if symbol_regex else None
 
-    while pwndbg.aglib.proc.alive:
+    while pwndbg.aglib.proc.alive():
         # Break on signal as it may be a segfault
-        if pwndbg.aglib.proc.stopped_with_signal:
+        if pwndbg.aglib.proc.stopped_with_signal():
             return
 
         ins = await break_next_branch(ec)
@@ -150,9 +183,9 @@ async def break_next_call(ec: pwndbg.dbg_mod.ExecutionController, symbol_regex=N
 
 
 async def break_next_ret(ec: pwndbg.dbg_mod.ExecutionController, address=None):
-    while pwndbg.aglib.proc.alive:
+    while pwndbg.aglib.proc.alive():
         # Break on signal as it may be a segfault
-        if pwndbg.aglib.proc.stopped_with_signal:
+        if pwndbg.aglib.proc.stopped_with_signal():
             return
 
         ins = await break_next_branch(ec, address)
@@ -160,7 +193,7 @@ async def break_next_ret(ec: pwndbg.dbg_mod.ExecutionController, address=None):
         if not ins:
             break
 
-        if capstone.CS_GRP_RET in ins.groups:
+        if CS_GRP_RET in ins.groups:
             return ins
 
 
@@ -175,25 +208,22 @@ async def break_on_next_matching_instruction(
         return False
 
     proc = pwndbg.dbg.selected_inferior()
-    while pwndbg.aglib.proc.alive:
+    while pwndbg.aglib.proc.alive():
         ins = next_matching_until_branch(mnemonic=mnemonic, op_str=op_str)
         if ins is not None:
             if ins.address != pwndbg.aglib.regs.pc:
-                print("Found instruction")
                 # Only set breakpoints at a different PC location, otherwise we
                 # will continue until we hit a breakpoint that's not related to
                 # this opeeration, or the program halts.
                 with proc.break_at(BreakpointLocation(ins.address), internal=True) as bp:
                     await ec.cont(bp)
                 return ins
-            else:
-                # We don't want to be spinning in place, nudge execution forward
-                # and try again.
-                pass
+            # We don't want to be spinning in place, nudge execution forward
+            # and try again.
+            pass
         else:
             # Move to the next branch instruction.
-            print("Moving to next branch")
-            nb = next_branch(pwndbg.aglib.regs.pc)
+            nb = next_branch(pwndbg.aglib.regs.pc, including_current=True)
             if nb is not None:
                 if nb.address != pwndbg.aglib.regs.pc:
                     # Stop right at the next branch instruction.
@@ -203,11 +233,11 @@ async def break_on_next_matching_instruction(
                     # Nudge execution so we take the branch we're on top of.
                     pass
 
-        if pwndbg.aglib.proc.alive:
+        if pwndbg.aglib.proc.alive():
             await ec.single_step()
 
         # Break on signal as it may be a segfault
-        if pwndbg.aglib.proc.stopped_with_signal:
+        if pwndbg.aglib.proc.stopped_with_signal():
             return False
 
     return False
@@ -219,7 +249,7 @@ async def break_on_program_code(ec: pwndbg.dbg_mod.ExecutionController) -> bool:
 
     :return: True for success, False when process ended or when pc is not at the code or if a signal occurred
     """
-    exe = pwndbg.aglib.proc.exe
+    exe = pwndbg.aglib.proc.exe()
     binary_exec_page_ranges = tuple(
         (p.start, p.end) for p in pwndbg.aglib.vmmap.get() if p.objfile == exe and p.execute
     )
@@ -230,14 +260,14 @@ async def break_on_program_code(ec: pwndbg.dbg_mod.ExecutionController) -> bool:
             print(message.error("The pc is already at the binary objfile code. Not stepping."))
             return False
 
-    proc = pwndbg.aglib.proc
     regs = pwndbg.aglib.regs
 
-    while proc.alive:
+    while pwndbg.aglib.proc.alive():
         # Break on signal as it may be a segfault
-        if proc.stopped_with_signal:
+        if pwndbg.aglib.proc.stopped_with_signal():
             return False
 
+        await break_next_ret(ec)
         await ec.single_step()
 
         for start, end in binary_exec_page_ranges:
@@ -249,7 +279,7 @@ async def break_on_program_code(ec: pwndbg.dbg_mod.ExecutionController) -> bool:
 
 async def break_on_next(ec: pwndbg.dbg_mod.ExecutionController, address=None) -> None:
     address = address or pwndbg.aglib.regs.pc
-    ins = pwndbg.aglib.disasm.one(address)
+    ins = pwndbg.aglib.disasm.disassembly.one(address)
 
     proc = pwndbg.dbg.selected_inferior()
     with proc.break_at(BreakpointLocation(ins.address + ins.size), internal=True) as bp:
