@@ -1,14 +1,16 @@
+"""
+Dereference and format pointer chains.
+"""
+
 from __future__ import annotations
 
-from typing import List
-
-import pwndbg.aglib.arch
+import pwndbg.aglib
 import pwndbg.aglib.memory
-import pwndbg.aglib.typeinfo
 import pwndbg.aglib.vmmap
-import pwndbg.color.memory as M
+import pwndbg.color.memory as mem_color
+import pwndbg.dbg_mod
+import pwndbg.dintegration
 import pwndbg.enhance
-import pwndbg.integration
 from pwndbg.color import ColorConfig
 from pwndbg.color import ColorParamSpec
 from pwndbg.color import theme
@@ -36,7 +38,7 @@ def get(
     hard_end: int = 0,
     include_start: bool = True,
     safe_linking: bool = False,
-) -> List[int] | None:
+) -> list[int] | None:
     """
     Recursively dereferences an address. For bare metal, it will stop when the address is not in any of vmmap pages to avoid redundant dereference.
 
@@ -59,6 +61,9 @@ def get(
     limit = int(limit)
 
     result = [address] if include_start else []
+
+    is_pagefault_supported = pwndbg.aglib.memory.is_pagefault_supported()
+
     for _ in range(limit):
         # Don't follow cycles, except to stop at the second occurrence.
         if result.count(address) >= 2:
@@ -71,16 +76,13 @@ def get(
         try:
             address = address + offset
 
-            # Avoid redundant dereferences in bare metal mode by checking
-            # if address is in any of vmmap pages
-            if not pwndbg.dbg.selected_inferior().is_linux() and not pwndbg.aglib.vmmap.find(
-                address
-            ):
+            # On embedded systems, it's non uncommon for MMIO regions to exist where memory reads might mutate the hardware/process state.
+            # This check prevents the memory dereferences to protect against this case.
+            # See discussion here: https://github.com/pwndbg/pwndbg/pull/385
+            if not is_pagefault_supported and not pwndbg.aglib.vmmap.find(address):
                 break
 
-            next_address = int(
-                pwndbg.aglib.memory.get_typed_pointer_value(pwndbg.aglib.typeinfo.ppvoid, address)
-            )
+            next_address = pwndbg.aglib.memory.read_pointer_width(address)
             address = next_address ^ ((address >> 12) if safe_linking else 0)
             address &= pwndbg.aglib.arch.ptrmask
             result.append(address)
@@ -98,7 +100,7 @@ config_contiguous = theme.add_param(
 
 
 def format(
-    value: int | List[int] | None,
+    value: int | list[int] | None,
     limit: int = LIMIT,
     code: bool = True,
     offset: int = 0,
@@ -138,8 +140,13 @@ def format(
     arrow_left = c.arrow(f" {config_arrow_left} ")
     arrow_right = c.arrow(f" {config_arrow_right} ")
 
+    # Ask the decompiler to resolve stack variables
+    stack_vars = pwndbg.dintegration.manager.get_stack_var_dict_all()
+
     # Colorize the chain
-    rest = [M.get_address_and_symbol(addr) if addr >= 0 else "" for addr in chain]
+    rest = [
+        mem_color.get_address_and_symbol(addr, stack_vars) if addr >= 0 else "" for addr in chain
+    ]
 
     # If the dereference limit is zero, skip any enhancements.
     if limit == 0:
