@@ -3,6 +3,7 @@ from __future__ import annotations
 import argparse
 
 import pwnlib
+import pwnlib.asm
 import pwnlib.context
 
 import pwndbg.commands
@@ -22,31 +23,6 @@ parser.add_argument(
     help="Target architecture",
 )
 
-parser.add_argument(
-    "-v",
-    "--avoid",
-    action="append",
-    help="Encode the shellcode to avoid the listed bytes (provided as hex)",
-)
-
-parser.add_argument(
-    "-n",
-    "--newline",
-    dest="avoid",
-    action="append_const",
-    const="0a",
-    help="Encode the shellcode to avoid newlines",
-)
-
-parser.add_argument(
-    "-z",
-    "--zero",
-    dest="avoid",
-    action="append_const",
-    const="00",
-    help="Encode the shellcode to avoid NULL bytes",
-)
-
 input_group = parser.add_mutually_exclusive_group(required=True)
 
 input_group.add_argument(
@@ -56,26 +32,20 @@ input_group.add_argument(
 input_group.add_argument("-i", "--infile", default=None, type=str, help="Specify input file")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, command_name="asm", category=CommandCategory.MISC)
-def asm(shellcode, format, arch, avoid, infile) -> None:
+@pwndbg.commands.Command(parser, command_name="asm", category=CommandCategory.MISC)
+def asm(shellcode: list[str], format: str, arch: str | None, infile: str) -> None:
     if infile:
         print(message.warn("Going to read from file: " + infile))
         with open(infile) as file:
             shellcode = [file.read()]
 
-    if not arch:
+    if arch is None:
         arch = pwnlib.context.context.arch
 
     bits_for_arch = pwnlib.context.context.architectures.get(arch, {}).get("bits")
     assembly = pwnlib.asm.asm(" ".join(shellcode), arch=arch, bits=bits_for_arch)
 
-    if avoid:
-        avoid = (str(byte) for byte in avoid)
-        avoid = pwnlib.unhex("".join(avoid))
-        print(message.warn("Going to avoid these bytes in hex: " + avoid.hex(" ")))
-        assembly = pwnlib.encode(assembly, avoid)
-
     if format == "hex":
-        assembly = assembly.hex()
-
-    print(assembly)
+        print(assembly.hex())
+    else:
+        print(assembly)
