@@ -13,7 +13,6 @@ from pwndbg.color import message
 from pwndbg.commands import CommandCategory
 
 parser = argparse.ArgumentParser(
-    formatter_class=argparse.RawTextHelpFormatter,
     description="""Set ignore-count of breakpoint number N to COUNT.
 
 While the ignore count is positive, execution will not stop on the breakpoint.
@@ -26,7 +25,7 @@ parser.add_argument(
 parser.add_argument("count", metavar="COUNT", type=int, help="The number to set COUNT.")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.BREAKPOINT)
+@pwndbg.commands.Command(parser, category=CommandCategory.BREAKPOINT)
 def ignore(bpnum, count) -> None:
     bps = gdb.breakpoints()
 
@@ -40,9 +39,9 @@ def ignore(bpnum, count) -> None:
         bp = next((bp for bp in bps if bp.number == bpnum), None)
 
         if bp is None:
-            print(message.error("No breakpoint number %d." % bpnum))
+            print(message.error(f"No breakpoint number {bpnum}."))
             return
 
     count = max(0, int(count))
     bp.ignore_count = count
-    print("Will ignore next %d crossings of breakpoint %d." % (count, bp.number))
+    print(f"Will ignore next {count} crossings of breakpoint {bp.number}.")
