@@ -4,7 +4,7 @@ import argparse
 from typing import NamedTuple
 
 import pwndbg.aglib.kernel
-import pwndbg.color.message as M
+import pwndbg.color.message as message
 import pwndbg.commands
 from pwndbg.commands import CommandCategory
 
@@ -103,27 +103,19 @@ _arch_hardening_options["aarch64"] = [
 parser = argparse.ArgumentParser(description="Checks for kernel hardening configuration options.")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.KERNEL)
+@pwndbg.commands.Command(parser, category=CommandCategory.KERNEL)
 @pwndbg.commands.OnlyWhenQemuKernel
 @pwndbg.commands.OnlyWhenPagingEnabled
 def kchecksec() -> None:
     kconfig = pwndbg.aglib.kernel.kconfig()
 
-    if not kconfig:
-        print(
-            M.warn(
-                "No kernel configuration found, make sure the kernel was built with CONFIG_IKCONFIG"
-            )
-        )
-        return
-
     options = _hardening_options + _arch_hardening_options.get(pwndbg.aglib.arch.name, [])
     for opt in options:
         config_name = opt.name
         val = kconfig.get(config_name)
-        color_func = M.error
+        color_func = message.error
         if (opt.desired and val) or (not opt.desired and not val):
-            color_func = M.success
+            color_func = message.success
 
         if val:
             print(color_func(f"CONFIG_{config_name} = {val}"))
