@@ -9,9 +9,9 @@ from pwndbg.commands import CommandCategory
 parser = argparse.ArgumentParser(description="Return the kernel commandline (/proc/cmdline).")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.KERNEL)
+@pwndbg.commands.Command(parser, category=CommandCategory.KERNEL)
 @pwndbg.commands.OnlyWhenQemuKernel
-@pwndbg.commands.OnlyWithKernelDebugSyms
+@pwndbg.commands.OnlyWithKernelDebugInfo
 @pwndbg.commands.OnlyWhenPagingEnabled
 def kcmdline() -> None:
     print(pwndbg.aglib.kernel.kcmdline())
