@@ -3,29 +3,23 @@ from __future__ import annotations
 import argparse
 
 import pwndbg.aglib.kernel
-import pwndbg.color.message as M
+import pwndbg.aglib.kernel.kconfig_mod
 import pwndbg.commands
 from pwndbg.commands import CommandCategory
 
-parser = argparse.ArgumentParser(
-    description="Outputs the kernel config (requires CONFIG_IKCONFIG)."
-)
+parser = argparse.ArgumentParser(description="Outputs the kernel config.")
 
 parser.add_argument("config_name", nargs="?", type=str, help="A config name to search for")
+parser.add_argument("-l", "--load", type=str, dest="file_path", help="load kernel config file")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.KERNEL)
+@pwndbg.commands.Command(parser, category=CommandCategory.KERNEL)
 @pwndbg.commands.OnlyWhenQemuKernel
 @pwndbg.commands.OnlyWhenPagingEnabled
-def kconfig(config_name=None) -> None:
+def kconfig(config_name=None, file_path=None) -> None:
     kconfig_ = pwndbg.aglib.kernel.kconfig()
-
-    if not kconfig_:
-        print(
-            M.warn(
-                "No kernel configuration found, make sure the kernel was built with CONFIG_IKCONFIG"
-            )
-        )
+    if file_path is not None:
+        kconfig_.update_with_file(file_path)
         return
 
     if config_name:
@@ -34,7 +28,7 @@ def kconfig(config_name=None) -> None:
             val = kconfig_[config_name]
             print(f"{key} = {val}")
         else:
-            key = pwndbg.lib.kernel.kconfig.config_to_key(config_name)
+            key = pwndbg.aglib.kernel.kconfig_mod.config_to_key(config_name)
             print(f"Config {key} not set")
     else:
         for name, val in kconfig_.items():
