@@ -1,7 +1,6 @@
 from __future__ import annotations
 
 import argparse
-from typing import Optional
 
 import pwndbg.aglib.kernel.nftables
 import pwndbg.commands
@@ -22,11 +21,11 @@ parser = argparse.ArgumentParser(
 parser.add_argument("nsid", type=int, nargs="?", help="Network Namespace ID")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.KERNEL)
+@pwndbg.commands.Command(parser, category=CommandCategory.KERNEL)
 @pwndbg.commands.OnlyWhenQemuKernel
-@pwndbg.commands.OnlyWithKernelDebugSyms
+@pwndbg.commands.OnlyWithKernelDebugInfo
 @pwndbg.commands.OnlyWhenPagingEnabled
-def knft_dump(nsid: Optional[int] = None):
+def knft_dump(nsid: int | None = None):
     nft = pwndbg.aglib.kernel.nftables.Nftables.find(nsid=nsid)
     if nft is None:
         print("No netns found")
@@ -41,11 +40,11 @@ parser = argparse.ArgumentParser(
 parser.add_argument("--nsid", "-n", type=int, help="Network Namespace ID")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.KERNEL)
+@pwndbg.commands.Command(parser, category=CommandCategory.KERNEL)
 @pwndbg.commands.OnlyWhenQemuKernel
-@pwndbg.commands.OnlyWithKernelDebugSyms
+@pwndbg.commands.OnlyWithKernelDebugInfo
 @pwndbg.commands.OnlyWhenPagingEnabled
-def knft_list_tables(nsid: Optional[int] = None):
+def knft_list_tables(nsid: int | None = None):
     nft = pwndbg.aglib.kernel.nftables.Nftables.find(nsid=nsid)
     if nft is None:
         print("No netns found")
@@ -66,12 +65,12 @@ parser.add_argument(
 parser.add_argument("table_name", nargs="?", type=str, help="Table name")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.KERNEL)
+@pwndbg.commands.Command(parser, category=CommandCategory.KERNEL)
 @pwndbg.commands.OnlyWhenQemuKernel
-@pwndbg.commands.OnlyWithKernelDebugSyms
+@pwndbg.commands.OnlyWithKernelDebugInfo
 @pwndbg.commands.OnlyWhenPagingEnabled
 def knft_list_chains(
-    table_family: Optional[int] = None, table_name: Optional[str] = None, nsid: Optional[int] = None
+    table_family: int | None = None, table_name: str | None = None, nsid: int | None = None
 ):
     is_any = False
     for nft in pwndbg.aglib.kernel.nftables.Table.find(
@@ -97,15 +96,15 @@ parser.add_argument("table_name", nargs="?", type=str, help="Table name")
 parser.add_argument("chain_name", nargs="?", type=str, help="Chain name")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.KERNEL)
+@pwndbg.commands.Command(parser, category=CommandCategory.KERNEL)
 @pwndbg.commands.OnlyWhenQemuKernel
-@pwndbg.commands.OnlyWithKernelDebugSyms
+@pwndbg.commands.OnlyWithKernelDebugInfo
 @pwndbg.commands.OnlyWhenPagingEnabled
 def knft_list_rules(
-    table_family: Optional[int] = None,
-    table_name: Optional[str] = None,
-    chain_name: Optional[str] = None,
-    nsid: Optional[int] = None,
+    table_family: int | None = None,
+    table_name: str | None = None,
+    chain_name: str | None = None,
+    nsid: int | None = None,
 ):
     is_any = False
     for nft in pwndbg.aglib.kernel.nftables.Chain.find(
@@ -131,16 +130,16 @@ parser.add_argument("chain_name", nargs="?", type=str, help="Chain name")
 parser.add_argument("rule_id", nargs="?", type=int, help="Rule Handle ID")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.KERNEL)
+@pwndbg.commands.Command(parser, category=CommandCategory.KERNEL)
 @pwndbg.commands.OnlyWhenQemuKernel
-@pwndbg.commands.OnlyWithKernelDebugSyms
+@pwndbg.commands.OnlyWithKernelDebugInfo
 @pwndbg.commands.OnlyWhenPagingEnabled
 def knft_list_exprs(
-    table_family: Optional[int] = None,
-    table_name: Optional[str] = None,
-    chain_name: Optional[str] = None,
-    rule_id: Optional[int] = None,
-    nsid: Optional[int] = None,
+    table_family: int | None = None,
+    table_name: str | None = None,
+    chain_name: str | None = None,
+    rule_id: int | None = None,
+    nsid: int | None = None,
 ):
     is_any = False
     for nft in pwndbg.aglib.kernel.nftables.Rule.find(
@@ -168,12 +167,12 @@ parser.add_argument(
 parser.add_argument("table_name", nargs="?", type=str, help="Table name")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.KERNEL)
+@pwndbg.commands.Command(parser, category=CommandCategory.KERNEL)
 @pwndbg.commands.OnlyWhenQemuKernel
-@pwndbg.commands.OnlyWithKernelDebugSyms
+@pwndbg.commands.OnlyWithKernelDebugInfo
 @pwndbg.commands.OnlyWhenPagingEnabled
 def knft_list_sets(
-    table_family: Optional[int] = None, table_name: Optional[str] = None, nsid: Optional[int] = None
+    table_family: int | None = None, table_name: str | None = None, nsid: int | None = None
 ):
     is_any = False
     for nft in pwndbg.aglib.kernel.nftables.Table.find(
@@ -197,12 +196,12 @@ parser.add_argument(
 parser.add_argument("table_name", nargs="?", type=str, help="Table name")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.KERNEL)
+@pwndbg.commands.Command(parser, category=CommandCategory.KERNEL)
 @pwndbg.commands.OnlyWhenQemuKernel
-@pwndbg.commands.OnlyWithKernelDebugSyms
+@pwndbg.commands.OnlyWithKernelDebugInfo
 @pwndbg.commands.OnlyWhenPagingEnabled
 def knft_list_objects(
-    table_family: Optional[int] = None, table_name: Optional[str] = None, nsid: Optional[int] = None
+    table_family: int | None = None, table_name: str | None = None, nsid: int | None = None
 ):
     is_any = False
     for nft in pwndbg.aglib.kernel.nftables.Table.find(
@@ -226,12 +225,12 @@ parser.add_argument(
 parser.add_argument("table_name", nargs="?", type=str, help="Table name")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.KERNEL)
+@pwndbg.commands.Command(parser, category=CommandCategory.KERNEL)
 @pwndbg.commands.OnlyWhenQemuKernel
-@pwndbg.commands.OnlyWithKernelDebugSyms
+@pwndbg.commands.OnlyWithKernelDebugInfo
 @pwndbg.commands.OnlyWhenPagingEnabled
 def knft_list_flowtables(
-    table_family: Optional[int] = None, table_name: Optional[str] = None, nsid: Optional[int] = None
+    table_family: int | None = None, table_name: str | None = None, nsid: int | None = None
 ):
     is_any = False
     for nft in pwndbg.aglib.kernel.nftables.Table.find(
