@@ -6,14 +6,15 @@ from __future__ import annotations
 
 import argparse
 import queue
-from typing import Dict
-from typing import List
 
 import pwndbg
+import pwndbg.aglib.memory
+import pwndbg.aglib.qemu
 import pwndbg.aglib.vmmap
-import pwndbg.color.memory as M
+import pwndbg.chain
+import pwndbg.color.memory as mem_color
 import pwndbg.commands
-from pwndbg.chain import c as C
+import pwndbg.dbg_mod
 from pwndbg.color import message
 from pwndbg.commands import CommandCategory
 
@@ -23,7 +24,7 @@ from pwndbg.commands import CommandCategory
 # visited_map is a map of children -> (parent,parent_start)
 def get_rec_addr_string(addr, visited_map):
     page = pwndbg.aglib.vmmap.find(addr)
-    arrow_right = C.arrow(" %s " % pwndbg.config.chain_arrow_right)
+    arrow_right = pwndbg.chain.c.arrow(f" {pwndbg.config.chain_arrow_right} ")
 
     if page is not None:
         if addr not in visited_map:
@@ -40,11 +41,10 @@ def get_rec_addr_string(addr, visited_map):
             return ""
         return (
             get_rec_addr_string(parent_base_addr, visited_map)
-            + M.get(parent_base_addr, text=curText)
+            + mem_color.get(parent_base_addr, text=curText)
             + arrow_right
         )
-    else:
-        return ""
+    return ""
 
 
 # Useful for debugging. Prints a map of child -> (parent, parent_start)
@@ -54,18 +54,18 @@ def dbg_print_map(maps) -> None:
 
 
 parser = argparse.ArgumentParser(
-    formatter_class=argparse.RawTextHelpFormatter,
     description="""
 Attempt to find a leak chain given a starting address.
-Scans memory near the given address, looks for pointers, and continues that process to attempt to find leaks.
 
-Example: leakfind $rsp --page_name=filename --max_offset=0x48 --max_depth=6. This would look for any chains of leaks \
-that point to a section in filename which begin near $rsp, are never 0x48 bytes further from a known pointer, \
-and are a maximum length of 6.
+Scans memory near the given address, looks for pointers, and continues that process to attempt to find leaks.
 """,
 )
 parser.add_argument(
-    "address", nargs="?", default="$sp", help="Starting address to find a leak chain from"
+    "address",
+    type=pwndbg.commands.AddressExpr,
+    nargs="?",
+    default="$sp",
+    help="Starting address to find a leak chain from",
 )
 parser.add_argument(
     "-p",
@@ -108,7 +108,16 @@ parser.add_argument(
 )
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.MEMORY)
+@pwndbg.commands.Command(
+    parser,
+    category=CommandCategory.MEMORY,
+    examples="""
+pwndbg> leakfind $rsp --page_name=filename --max_offset=0x48 --max_depth=6.
+This would look for any chains of leaks that point to a section in filename
+which begin near $rsp, are never 0x48 bytes further from a known pointer,
+and are a maximum length of 6.
+""",
+)
 @pwndbg.commands.OnlyWhenRunning
 def leakfind(
     address=None,
@@ -119,17 +128,12 @@ def leakfind(
     negative_offset: int = 0x0,
 ):
     if address is None:
-        raise argparse.ArgumentTypeError("No starting address provided.")
-
-    address = int(address)
-
-    foundPages = pwndbg.aglib.vmmap.find(address)
-
-    if not foundPages:
-        raise argparse.ArgumentTypeError("Starting address is not mapped.")
+        print("No start address provided.")
+        return
 
     if not pwndbg.aglib.memory.peek(address):
-        raise argparse.ArgumentTypeError("Unable to read from starting address.")
+        print("Unable to read from starting address.")
+        return
 
     # Just warn the user that a large depth might be slow.
     # Probably worth checking offset^depth < threshold. Do this when more benchmarking is established.
@@ -143,9 +147,9 @@ def leakfind(
     # parent_start_address is an address that a previous address pointed to.
     # We need to store both so that we can nicely create our leak chain.
     visited_map = {}
-    visited_set = {int(address)}
-    address_queue: "queue.Queue[int]" = queue.Queue()
-    address_queue.put(int(address))
+    visited_set = {address}
+    address_queue: queue.Queue[int] = queue.Queue()
+    address_queue.put(address)
     depth = 0
     time_to_depth_increase = 0
 
@@ -163,7 +167,7 @@ def leakfind(
         ):
             try:
                 cur_addr &= pwndbg.aglib.arch.ptrmask
-                result = int(pwndbg.aglib.memory.pvoid(cur_addr))
+                result = int(pwndbg.aglib.memory.read_pointer_width(cur_addr))
                 if result in visited_map or result in visited_set:
                     continue
                 visited_map[result] = (
@@ -177,8 +181,8 @@ def leakfind(
                 break
 
     # A map of length->list of lines. Used to let us print in a somewhat nice manner.
-    output_map: Dict[int, List[str]] = {}
-    arrow_right = C.arrow(" %s " % pwndbg.config.chain_arrow_right)
+    output_map: dict[int, list[str]] = {}
+    arrow_right = pwndbg.chain.c.arrow(f" {pwndbg.config.chain_arrow_right} ")
 
     for child in visited_map:
         child_page = pwndbg.aglib.vmmap.find(child)
@@ -187,9 +191,9 @@ def leakfind(
                 continue
             line = (
                 get_rec_addr_string(child, visited_map)
-                + M.get(child)
+                + mem_color.get(child)
                 + " "
-                + M.get(child, text=child_page.objfile)
+                + mem_color.get(child, text=child_page.objfile)
             )
             chain_length = line.count(arrow_right)
             if chain_length in output_map:
