@@ -2,9 +2,10 @@ from __future__ import annotations
 
 import argparse
 
+import pwndbg.aglib.dynamic
+import pwndbg.aglib.proc
 import pwndbg.color as color
-import pwndbg.gdblib.dynamic
-import pwndbg.gdblib.proc
+import pwndbg.commands
 from pwndbg.commands import CommandCategory
 
 parser = argparse.ArgumentParser(
@@ -12,18 +13,18 @@ parser = argparse.ArgumentParser(
 )
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.LINUX)
+@pwndbg.commands.Command(parser, category=CommandCategory.LINUX)
 @pwndbg.commands.OnlyWhenRunning
 def linkmap() -> None:
     is_first = True
     rows = [["Node", "Objfile", "Load Bias", "Dynamic Segment"]]
-    for obj in pwndbg.gdblib.dynamic.link_map():
+    for obj in pwndbg.aglib.dynamic.link_map():
         name = obj.name().decode("utf-8")
         if name == "":
             name = "<Unknown"
             if is_first:
                 is_first = False
-                name += f", likely {pwndbg.gdblib.proc.exe}"
+                name += f", likely {pwndbg.aglib.proc.exe()}"
             name += ">"
         rows.append(
             [f"{obj.link_map_address:#x}", name, f"{obj.load_bias():#x}", f"{obj.dynamic():#x}"]
