@@ -2,30 +2,19 @@ from __future__ import annotations
 
 import argparse
 
-import pwndbg.aglib.file
 import pwndbg.aglib.shellcode
-import pwndbg.chain
 import pwndbg.commands
-import pwndbg.enhance
+import pwndbg.dbg_mod
 import pwndbg.lib.memory
-import pwndbg.wrappers.checksec
-import pwndbg.wrappers.readelf
 from pwndbg.commands import CommandCategory
 
 parser = argparse.ArgumentParser(
-    formatter_class=argparse.RawTextHelpFormatter,
     description="""
 Calls the mprotect syscall and prints its result value.
 
 Note that the mprotect syscall may fail for various reasons
 (see `man mprotect`) and a non-zero error return value
 can be decoded with the `errno <value>` command.
-
-Examples:
-    mprotect $rsp 4096 PROT_READ|PROT_WRITE|PROT_EXEC
-    mprotect $rsp 4096 rwx
-    mprotect $rsp 4096 7
-    mprotect some_symbol 0x1000 PROT_NONE
 """,
 )
 parser.add_argument(
@@ -64,7 +53,7 @@ def prot_str_to_val(protstr: str) -> int:
             if k in protstr:
                 prot_int |= v
         return prot_int
-    elif all(x in "RWX" for x in protstr):
+    if all(x in "RWX" for x in protstr):
         prot_int = 0
         for c in protstr:
             if c == "R":
@@ -74,11 +63,10 @@ def prot_str_to_val(protstr: str) -> int:
             elif c == "X":
                 prot_int |= 4
         return prot_int
-    else:
-        try:
-            return int(protstr, 0)
-        except ValueError:
-            raise ValueError("Invalid protection string passed into mprotect")
+    try:
+        return int(protstr, 0)
+    except ValueError:
+        raise ValueError("Invalid protection string passed into mprotect")
 
 
 def prot_val_to_str(protval: int) -> str:
@@ -91,7 +79,16 @@ def prot_val_to_str(protval: int) -> str:
     return "|".join(ret)
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.MEMORY)
+@pwndbg.commands.Command(
+    parser,
+    category=CommandCategory.MEMORY,
+    examples="""
+mprotect $rsp 4096 PROT_READ|PROT_WRITE|PROT_EXEC
+mprotect $rsp 4096 rwx
+mprotect $rsp 4096 7
+mprotect some_symbol 0x1000 PROT_NONE
+""",
+)
 @pwndbg.commands.OnlyWhenRunning
 def mprotect(addr, length, prot) -> None:
     prot_int = prot_str_to_val(prot)
