@@ -7,7 +7,10 @@ from __future__ import annotations
 import argparse
 
 import pwndbg.aglib.next
+import pwndbg.aglib.proc
 import pwndbg.commands
+import pwndbg.commands.context
+import pwndbg.dbg_mod
 from pwndbg.commands import CommandCategory
 
 
@@ -20,7 +23,7 @@ async def _nextjmp(ec: pwndbg.dbg_mod.ExecutionController):
         pwndbg.commands.context.context()
 
 
-@pwndbg.commands.ArgparsedCommand(
+@pwndbg.commands.Command(
     "Breaks at the next jump instruction.", aliases=["nextjump"], category=CommandCategory.NEXT
 )
 @pwndbg.commands.OnlyWhenRunning
@@ -40,7 +43,7 @@ parser.add_argument(
 )
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.NEXT)
+@pwndbg.commands.Command(parser, category=CommandCategory.NEXT)
 @pwndbg.commands.OnlyWhenRunning
 def nextcall(symbol_regex=None) -> None:
     """Breaks at the next call instruction"""
@@ -60,9 +63,7 @@ async def _nextret(ec: pwndbg.dbg_mod.ExecutionController):
         pwndbg.commands.context.context()
 
 
-@pwndbg.commands.ArgparsedCommand(
-    "Breaks at next return-like instruction.", category=CommandCategory.NEXT
-)
+@pwndbg.commands.Command("Breaks at next return-like instruction.", category=CommandCategory.NEXT)
 @pwndbg.commands.OnlyWhenRunning
 def nextret() -> None:
     """Breaks at next return-like instruction"""
@@ -75,7 +76,7 @@ async def _stepret(ec: pwndbg.dbg_mod.ExecutionController):
     Execution controller for the `stepret` command.
     """
     while (
-        pwndbg.aglib.proc.alive
+        pwndbg.aglib.proc.alive()
         and not (await pwndbg.aglib.next.break_next_ret(ec))
         and (await pwndbg.aglib.next.break_next_branch(ec))
     ):
@@ -85,7 +86,7 @@ async def _stepret(ec: pwndbg.dbg_mod.ExecutionController):
         continue
 
 
-@pwndbg.commands.ArgparsedCommand(
+@pwndbg.commands.Command(
     "Breaks at next return-like instruction by 'stepping' to it.", category=CommandCategory.NEXT
 )
 @pwndbg.commands.OnlyWhenRunning
@@ -102,7 +103,7 @@ async def _nextproginstr(ec: pwndbg.dbg_mod.ExecutionController):
     await pwndbg.aglib.next.break_on_program_code(ec)
 
 
-@pwndbg.commands.ArgparsedCommand(
+@pwndbg.commands.Command(
     "Breaks at the next instruction that belongs to the running program.",
     category=CommandCategory.NEXT,
 )
@@ -115,7 +116,7 @@ parser = argparse.ArgumentParser(description="Breaks on the instruction after th
 parser.add_argument("addr", type=int, default=None, nargs="?", help="The address to break after.")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, aliases=["so"], category=CommandCategory.NEXT)
+@pwndbg.commands.Command(parser, aliases=["so"], category=CommandCategory.NEXT)
 @pwndbg.commands.OnlyWhenRunning
 def stepover(addr=None) -> None:
     """Sets a breakpoint on the instruction after this one"""
@@ -134,14 +135,14 @@ async def _nextsyscall(ec: pwndbg.dbg_mod.ExecutionController):
     Execution controller for the `nextsyscall` command
     """
     while (
-        pwndbg.aglib.proc.alive
+        pwndbg.aglib.proc.alive()
         and not (await pwndbg.aglib.next.break_next_interrupt(ec))
         and (await pwndbg.aglib.next.break_next_branch(ec))
     ):
         continue
 
 
-@pwndbg.commands.ArgparsedCommand(
+@pwndbg.commands.Command(
     "Breaks at the next syscall not taking branches.",
     aliases=["nextsc"],
     category=CommandCategory.NEXT,
@@ -160,9 +161,9 @@ async def _stepsyscall(ec: pwndbg.dbg_mod.ExecutionController):
     """
 
     while (
-        pwndbg.aglib.proc.alive
-        and not (await pwndbg.aglib.next.break_next_interrupt(ec))
-        and (await pwndbg.aglib.next.break_next_branch(ec))
+        pwndbg.aglib.proc.alive()
+        and not (await pwndbg.aglib.next.break_next_interrupt(ec, honor_current_branch=True))
+        and (await pwndbg.aglib.next.break_next_branch(ec, including_current=True))
     ):
         # Here we are e.g. on a CALL instruction (temporarily breakpointed by `break_next_branch`)
         # We need to step so that we take this branch instead of ignoring it
@@ -170,7 +171,7 @@ async def _stepsyscall(ec: pwndbg.dbg_mod.ExecutionController):
         continue
 
 
-@pwndbg.commands.ArgparsedCommand(
+@pwndbg.commands.Command(
     "Breaks at the next syscall by taking branches.",
     aliases=["stepsc"],
     category=CommandCategory.NEXT,
@@ -194,7 +195,7 @@ parser.add_argument(
 )
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.NEXT)
+@pwndbg.commands.Command(parser, category=CommandCategory.NEXT)
 @pwndbg.commands.OnlyWhenRunning
 def stepuntilasm(mnemonic, op_str) -> None:
     if len(op_str) == 0:
