@@ -1,23 +1,20 @@
 from __future__ import annotations
 
 import argparse
-from typing import Dict
-from typing import Tuple
-
-from pwnlib.asm import asm
-from pwnlib.asm import disasm
 
+import pwndbg.aglib.asm
 import pwndbg.aglib.memory
 import pwndbg.color.context
 import pwndbg.color.memory
 import pwndbg.color.syntax_highlight
 import pwndbg.commands
 import pwndbg.lib.cache
+from pwndbg.aglib.disasm.disassembly import get_disassembler
 from pwndbg.color import message
 from pwndbg.commands import CommandCategory
 
 # Keep old patches made so we can revert them
-patches: Dict[int, Tuple[bytearray, bytearray]] = {}
+patches: dict[int, tuple[bytes, bytes]] = {}
 
 
 parser = argparse.ArgumentParser(description="Patches given instruction with given code or bytes.")
@@ -26,10 +23,10 @@ parser.add_argument("ins", type=str, help="instruction[s]")
 parser.add_argument("-q", "--quiet", action="store_true", help="don't print anything")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.MISC)
+@pwndbg.commands.Command(parser, category=CommandCategory.MISC)
 @pwndbg.commands.OnlyWhenRunning
 def patch(address: int, ins: str, quiet: bool) -> None:
-    new_mem = asm(ins)
+    new_mem = pwndbg.aglib.asm.asm(ins)
 
     old_mem = pwndbg.aglib.memory.read(address, len(new_mem))
 
@@ -48,7 +45,7 @@ parser2 = argparse.ArgumentParser(description="Revert patch at given address.")
 parser2.add_argument("address", type=int, help="Address to revert patch on")
 
 
-@pwndbg.commands.ArgparsedCommand(parser2, category=CommandCategory.MISC)
+@pwndbg.commands.Command(parser2, category=CommandCategory.MISC)
 @pwndbg.commands.OnlyWhenRunning
 def patch_revert(address: int) -> None:
     if not patches:
@@ -58,14 +55,14 @@ def patch_revert(address: int) -> None:
     if address == -1:
         for addr, (old, _new) in patches.items():
             pwndbg.aglib.memory.write(addr, old)
-            print(message.notice("Reverted patch at %#x" % addr))
+            print(message.notice(f"Reverted patch at {addr:#x}"))
         patches.clear()
     elif address in patches:
         old, _new = patches.pop(address)
         pwndbg.aglib.memory.write(address, old)
-        print(message.notice("Reverted patch at %#x" % address))
+        print(message.notice(f"Reverted patch at {address:#x}"))
     else:
-        print(message.error("Address %#x not found in patch list" % address))
+        print(message.error(f"Address {address:#x} not found in patch list"))
 
     pwndbg.lib.cache.clear_caches()
 
@@ -73,7 +70,7 @@ def patch_revert(address: int) -> None:
 parser3 = argparse.ArgumentParser(description="List all patches.")
 
 
-@pwndbg.commands.ArgparsedCommand(parser3, category=CommandCategory.MISC)
+@pwndbg.commands.Command(parser3, category=CommandCategory.MISC)
 @pwndbg.commands.OnlyWhenRunning
 def patch_list() -> None:
     if not patches:
@@ -82,8 +79,14 @@ def patch_list() -> None:
 
     print(pwndbg.color.context.banner("Patches:"))
     for addr, (old, new) in patches.items():
-        old_insns = disasm(old, byte=False, offset=False)
-        new_insns = disasm(new, byte=False, offset=False)
+        cs = get_disassembler(pwndbg.aglib.arch.get_capstone_constants(addr))
+
+        old_insns = "\n".join(
+            [f"{x.mnemonic} {x.op_str}".strip() for x in cs.disasm(old, offset=addr)]
+        )
+        new_insns = "\n".join(
+            [f"{x.mnemonic} {x.op_str}".strip() for x in cs.disasm(new, offset=addr)]
+        )
 
         colored_addr = pwndbg.color.memory.get(addr)
 
