@@ -8,13 +8,11 @@ import pwndbg.aglib.memory
 import pwndbg.aglib.proc
 import pwndbg.auxv
 import pwndbg.commands
-import pwndbg.commands.context
-import pwndbg.commands.telescope
 from pwndbg.color import message
 from pwndbg.commands import CommandCategory
 
 
-@pwndbg.commands.ArgparsedCommand("Gets the current file.", category=CommandCategory.MISC)
+@pwndbg.commands.Command("Gets the current file.", category=CommandCategory.MISC)
 @pwndbg.commands.OnlyWhenRunning
 def getfile() -> None:
     print(repr(pwndbg.auxv.get().AT_EXECFN))
@@ -24,16 +22,16 @@ parser = argparse.ArgumentParser(description="Continue execution until an addres
 parser.add_argument("target", type=int, help="Location to stop execution at")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.NEXT)
+@pwndbg.commands.Command(parser, category=CommandCategory.NEXT)
 def xuntil(target) -> None:
     try:
         addr = target
 
         if not pwndbg.aglib.memory.peek(addr):
-            print(message.error("Invalid address %#x" % addr))
+            print(message.error(f"Invalid address {addr:#x}"))
             return
 
-        spec = "*%#x" % (addr)
+        spec = f"*{addr:#x}"
     except (TypeError, ValueError):
         # The following gdb command will throw an error if the symbol is not defined.
         try:
@@ -44,7 +42,7 @@ def xuntil(target) -> None:
         spec = target
 
     gdb.Breakpoint(spec, temporary=True)
-    if pwndbg.aglib.proc.alive:
+    if pwndbg.aglib.proc.alive():
         gdb.execute("continue", from_tty=False)
     else:
         gdb.execute("run", from_tty=False)
