@@ -1,18 +1,17 @@
 from __future__ import annotations
 
 import argparse
-from typing import Optional
 
+import pwndbg.aglib
 import pwndbg.aglib.memory
 import pwndbg.aglib.symbol
 import pwndbg.chain
 import pwndbg.commands
-import pwndbg.dbg
+import pwndbg.dbg_mod
 from pwndbg.color import message
 from pwndbg.commands import CommandCategory
 
 parser = argparse.ArgumentParser(
-    formatter_class=argparse.RawTextHelpFormatter,
     description="""Dumps the elements of a linked list.
 
 This command traverses the linked list beginning at a given element, dumping its
@@ -187,15 +186,15 @@ parser.add_argument(
 )
 
 
-@pwndbg.commands.ArgparsedCommand(parser, command_name="plist", category=CommandCategory.MISC)
+@pwndbg.commands.Command(parser, command_name="plist", category=CommandCategory.MISC)
 def plist(
     path: str,
     next: str,
     sentinel: int,
-    inner_name: Optional[str],
-    field_name: Optional[str],
+    inner_name: str | None,
+    field_name: str | None,
     offset: int,
-    count: Optional[int] = None,
+    count: int | None = None,
 ) -> None:
     # Have GDB parse the path for us and check if it's valid.
     try:
@@ -273,8 +272,21 @@ def plist(
     if next_ptr.is_optimized_out:
         print(message.error(f"{path}{sep}{next_ptr_name} has been optimized out"))
         return
-    if next_ptr.type.code != pwndbg.dbg_mod.TypeCode.POINTER:
-        print(message.error(f"{path}{sep}{next_ptr_name} is not a pointer"))
+
+    # Check if the field is a pointer type, or an integer type with pointer size
+    # (e.g., size_t, uintptr_t, unsigned long on 64-bit systems)
+    # Strip typedefs to get the underlying type (e.g., size_t -> unsigned long)
+    underlying_type = next_ptr.type.strip_typedefs()
+    is_pointer_type = underlying_type.code == pwndbg.dbg_mod.TypeCode.POINTER
+    is_pointer_sized_int = (
+        underlying_type.code == pwndbg.dbg_mod.TypeCode.INT
+        and underlying_type.sizeof == pwndbg.aglib.arch.ptrsize
+    )
+
+    if not (is_pointer_type or is_pointer_sized_int):
+        print(
+            message.error(f"{path}{sep}{next_ptr_name} is not a pointer or pointer-sized integer")
+        )
         return
 
     # If the user wants a specific field to be displayed, resolve it.
@@ -360,7 +372,17 @@ def plist(
     # Here, we figure out how many bytes to subtract from *typeof(inner) so that
     # we can have a *typeof(first).
     pointee_offset = 0
-    if inner is not None and next_ptr.type.target() == inner.type:
+
+    # For pointer-sized integers (like size_t), we can't validate the target type
+    # since they don't have a target() method. We assume they point to the outer structure.
+    if is_pointer_sized_int:
+        # For integer types, we assume pointee_offset is 0 (pointing to outer structure)
+        # unless we have an inner structure, in which case the user needs to use --inner
+        # and we can't automatically determine the offset.
+        if inner is not None:
+            # We can't determine if it points to inner or outer type, so assume outer
+            pointee_offset = 0
+    elif inner is not None and next_ptr.type.target() == inner.type:
         pointee_offset = inner_offset
     elif next_ptr.type.target() == first.type:
         # We've already got everything we need for this mode.
