@@ -67,7 +67,7 @@ capabilities = {
 }
 
 
-def tcp():
+def tcp(tid: int):
     # For reference, see:
     # https://www.kernel.org/doc/Documentation/networking/proc_net_tcp.txt
     """
@@ -75,29 +75,34 @@ def tcp():
     TCP connections. A typical entry of /proc/net/tcp would look like this (split
     up into 3 parts because of the length of the line):
     """
-    data = pwndbg.aglib.file.get("/proc/net/tcp").decode()
-    return pwndbg.lib.net.tcp(data)
+    data = pwndbg.aglib.file.get(f"/proc/{tid}/net/tcp").decode()
+    return pwndbg.lib.net.tcp(data, pwndbg.aglib.arch.endian)
 
 
-def unix():
+def tcp6(tid: int):
+    data = pwndbg.aglib.file.get(f"/proc/{tid}/net/tcp6").decode()
+    return pwndbg.lib.net.tcp6(data, pwndbg.aglib.arch.endian)
+
+
+def unix(tid: int):
     # We use errors=ignore because of https://github.com/pwndbg/pwndbg/issues/1544
     # TODO/FIXME: this may not be the best solution because we may end up with
     # invalid UDS data. Can this be a problem?
-    data = pwndbg.aglib.file.get("/proc/net/unix").decode(errors="ignore")
+    data = pwndbg.aglib.file.get(f"/proc/{tid}/net/unix").decode(errors="ignore")
     return pwndbg.lib.net.unix(data)
 
 
-def netlink():
-    data = pwndbg.aglib.file.get("/proc/net/netlink").decode()
+def netlink(tid: int):
+    data = pwndbg.aglib.file.get(f"/proc/{tid}/net/netlink").decode()
     return pwndbg.lib.net.netlink(data)
 
 
 class Process:
     def __init__(self, pid=None, tid=None) -> None:
         if pid is None:
-            pid = pwndbg.aglib.proc.pid
+            pid = pwndbg.aglib.proc.pid()
         if tid is None:
-            tid = pwndbg.aglib.proc.tid
+            tid = pwndbg.aglib.proc.tid()
         if not tid:
             tid = pid
         self.pid = pid
@@ -106,7 +111,7 @@ class Process:
     @property
     @pwndbg.lib.cache.cache_until("stop")
     def selinux(self) -> str:
-        path = "/proc/%i/task/%i/attr/current" % (self.pid, self.tid)
+        path = f"/proc/{self.pid}/task/{self.tid}/attr/current"
         try:
             raw = pwndbg.aglib.file.get(path)
             return raw.decode().rstrip("\x00").strip()
@@ -129,7 +134,7 @@ class Process:
     @property
     @pwndbg.lib.cache.cache_until("stop")
     def status(self):
-        raw = pwndbg.aglib.file.get("/proc/%i/task/%i/status" % (self.pid, self.tid))
+        raw = pwndbg.aglib.file.get(f"/proc/{self.pid}/task/{self.pid}/status")
 
         status = {}
         for line in raw.splitlines():
@@ -187,7 +192,7 @@ class Process:
         fds = {}
 
         for i in range(self.fdsize):
-            link = pwndbg.aglib.file.readlink("/proc/%i/fd/%i" % (pwndbg.aglib.proc.pid, i))
+            link = pwndbg.aglib.file.readlink(f"/proc/{pwndbg.aglib.proc.pid()}/fd/{i}")
 
             if link:
                 fds[i] = link
@@ -203,7 +208,7 @@ class Process:
         socket = "socket:["
         result = []
 
-        functions = [tcp, unix, netlink]
+        functions = [tcp, tcp6, unix, netlink]
 
         for fd, path in fds.items():
             if socket not in path:
@@ -213,7 +218,7 @@ class Process:
             inode = int(inode)
 
             for func in functions:
-                for x in func():
+                for x in func(self.tid):
                     if x.inode == inode:
                         x.fd = fd
                         result.append(x)
@@ -221,15 +226,13 @@ class Process:
         return tuple(result)
 
 
-@pwndbg.commands.ArgparsedCommand(
-    "Gets the pid.", aliases=["getpid"], category=CommandCategory.PROCESS
-)
+@pwndbg.commands.Command("Gets the pid.", aliases=["getpid"], category=CommandCategory.PROCESS)
 @pwndbg.commands.OnlyWhenRunning
 def pid() -> None:
-    print(pwndbg.aglib.proc.pid)
+    print(pwndbg.aglib.proc.pid())
 
 
-@pwndbg.commands.ArgparsedCommand(
+@pwndbg.commands.Command(
     "Display information about the running process.", category=CommandCategory.PROCESS
 )
 @pwndbg.commands.OnlyWhenRunning
@@ -246,7 +249,7 @@ def procinfo() -> None:
             )
         )
     exe = pwndbg.auxv.get().AT_EXECFN
-    print("%-10s %r" % ("exe", exe))
+    print(f"{'exe':<10} {exe!r}")
 
     proc = Process()
 
@@ -254,32 +257,28 @@ def procinfo() -> None:
     if not proc.status:
         return
 
-    print("%-10s %s" % ("cmdline", proc.cmdline))
-
-    print("%-10s %s" % ("cwd", proc.cwd))
+    print(f"{'cmdline':<10} {proc.cmdline}")
+    print(f"{'cwd':<10} {proc.cwd}")
 
     files = dict(proc.open_files)
 
     for c in proc.connections:
         files[c.fd] = str(c)
 
-    print("%-10s %s" % ("pid", proc.pid))
-    print("%-10s %s" % ("tid", proc.tid))
+    print(f"{'pid':<10} {proc.pid}")
+    print(f"{'tid':<10} {proc.tid}")
 
-    # Indicate if the process is restricted by SELinux policies
-    if proc.selinux != "" and proc.selinux != "unconfined":
-        print("%-10s %s" % ("selinux", proc.selinux))
+    if proc.selinux and proc.selinux != "unconfined":
+        print(f"{'selinux':<10} {proc.selinux}")
 
-    print("%-10s %s" % ("ppid", proc.ppid))
-
-    print("%-10s %s" % ("uid", proc.uid))
-    print("%-10s %s" % ("gid", proc.gid))
-    print("%-10s %s" % ("groups", proc.groups))
+    print(f"{'ppid':<10} {proc.ppid}")
+    print(f"{'uid':<10} {proc.uid}")
+    print(f"{'gid':<10} {proc.gid}")
+    print(f"{'groups':<10} {proc.groups}")
 
     for fd, path in files.items():
         if not set(path) < set(string.printable):
             path = repr(path)
-
-        print("%-10s %s" % ("fd[%i]" % fd, path))
+        print(f"{f'fd[{fd}]':<10} {path}")
 
     return
