@@ -6,19 +6,27 @@ from __future__ import annotations
 
 import argparse
 
+import pwndbg.commands
 import pwndbg.profiling
 from pwndbg.commands import CommandCategory
 
-parser = argparse.ArgumentParser(description="Utilities for profiling pwndbg.")
+parser = argparse.ArgumentParser(
+    description="""
+Utilities for profiling Pwndbg.
+
+Check out the `./profiling` folder for other useful utilities.
+Use `./profiling/print_stats.py` to generate a report from a `.pstats` file.
+"""
+)
 subparsers = parser.add_subparsers(dest="command")
-parser_start = subparsers.add_parser("start", prog="profiler start")
-parser_stop = subparsers.add_parser("stop", prog="profiler stop")
+parser_start = subparsers.add_parser("start", description="Starts the profiler.")
+parser_stop = subparsers.add_parser("stop", description="Stop the profiler and save the results.")
 parser_stop.add_argument(
     "--file", type=str, default="pwndbg.pstats", help="Output file for profile data."
 )
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.PWNDBG)
+@pwndbg.commands.Command(parser, category=CommandCategory.PWNDBG)
 def profiler(command, file="pwndbg.pstats") -> None:
     if command == "start":
         print("Starting profiler.")
