@@ -3,9 +3,6 @@ from __future__ import annotations
 import argparse
 import ctypes
 from string import printable
-from typing import Dict
-from typing import List
-from typing import Set
 
 from tabulate import tabulate
 
@@ -14,14 +11,16 @@ import pwndbg.aglib.heap
 import pwndbg.aglib.memory
 import pwndbg.aglib.proc
 import pwndbg.aglib.symbol
-import pwndbg.aglib.typeinfo
 import pwndbg.aglib.vmmap
 import pwndbg.chain
-import pwndbg.color.context as C
-import pwndbg.color.memory as M
+import pwndbg.color.context as ctx_color
+import pwndbg.color.memory as mem_color
 import pwndbg.commands
-import pwndbg.glibc
-import pwndbg.lib.heap.helpers
+import pwndbg.commands.hexdump
+import pwndbg.dbg_mod
+import pwndbg.lib.memory
+import pwndbg.libc
+import pwndbg.libc.glibc
 from pwndbg.aglib.heap import heap_chain_limit
 from pwndbg.aglib.heap.ptmalloc import Arena
 from pwndbg.aglib.heap.ptmalloc import Bins
@@ -33,10 +32,9 @@ from pwndbg.aglib.heap.ptmalloc import Heap
 from pwndbg.color import generateColorFunction
 from pwndbg.color import message
 from pwndbg.commands import CommandCategory
-from pwndbg.commands.config import display_config
 
 
-def read_chunk(addr: int) -> Dict[str, int]:
+def read_chunk(addr: int) -> dict[str, int]:
     """Read a chunk's metadata."""
     # In GLIBC versions <= 2.24 the `mchunk_[prev_]size` field was named `[prev_]size`.
     # To support both versions, change the new names to the old ones here so that
@@ -53,36 +51,39 @@ def read_chunk(addr: int) -> Dict[str, int]:
         )
     else:
         val = pwndbg.aglib.heap.current.malloc_chunk(addr)
-    value_keys: List[str] = val.type.keys()
+    value_keys: list[str] = val.type.keys()
     return {renames.get(key, key): int(val[key]) for key in value_keys}
 
 
-def format_bin(bins: Bins, verbose: bool = False, offset: int | None = None) -> List[str]:
+def format_bin(bins: Bins, verbose: bool = False, offset: int | None = None) -> list[str]:
     assert isinstance(pwndbg.aglib.heap.current, GlibcMemoryAllocator)
+    assert pwndbg.libc.which() == pwndbg.libc.LibcType.GLIBC
     allocator = pwndbg.aglib.heap.current
     if offset is None:
         offset = allocator.chunk_key_offset("fd")
 
-    result: List[str] = []
+    result: list[str] = []
     bins_type = bins.bin_type
 
+    version = pwndbg.libc.version()
+
     for size in bins.bins:
         b = bins.bins[size]
         count: int | None = None
-        chain_fd: List[int] = []
-        chain_bk: List[int] | None = []
+        chain_fd: list[int] = []
+        chain_bk: list[int] | None = []
         is_chain_corrupted = False
         safe_lnk = False
 
         # fastbins consists of only single linked list
         if bins_type == BinType.FAST:
             chain_fd = b.fd_chain
-            safe_lnk = pwndbg.glibc.check_safe_linking()
+            safe_lnk = pwndbg.libc.glibc.check_safe_linking(version)
         # tcachebins consists of single linked list and entries count
         elif bins_type == BinType.TCACHE:
             chain_fd = b.fd_chain
             count = b.count
-            safe_lnk = pwndbg.glibc.check_safe_linking()
+            safe_lnk = pwndbg.libc.glibc.check_safe_linking(version)
         # normal bins consists of double linked list and may be corrupted (we can detect corruption)
         else:  # normal bin
             chain_fd = b.fd_chain
@@ -123,7 +124,7 @@ def format_bin(bins: Bins, verbose: bool = False, offset: int | None = None) ->
             )
         else:
             if count is not None:
-                line = (message.hint(size) + message.hint(" [%3d]" % count) + ": ").ljust(13)
+                line = (message.hint(size) + message.hint(f" [{count:3d}]") + ": ").ljust(13)
             else:
                 line = (message.hint(size) + ": ").ljust(13)
             line += formatted_chain
@@ -136,26 +137,19 @@ def format_bin(bins: Bins, verbose: bool = False, offset: int | None = None) ->
     return result
 
 
-def print_no_arena_found_error(tid=None) -> None:
+def print_no_arena_found_error(tid: int | None = None) -> None:
     if tid is None:
-        tid = pwndbg.aglib.proc.thread_id
+        tid = pwndbg.aglib.proc.thread_id()
     print(
         message.notice(
             f"No arena found for thread {message.hint(tid)} (the thread hasn't performed any allocations)."
         )
     )
-    if pwndbg.dbg.name() == pwndbg.dbg_mod.DebuggerType.GDB:
-        print(
-            message.hint(
-                "Use `set libthread-db-search-path <path>` if the path contains a compatible libthread_db.\n"
-                "Then rerun the program."
-            )
-        )
 
 
 def print_no_tcache_bins_found_error(tid: int | None = None) -> None:
     if tid is None:
-        tid = pwndbg.aglib.proc.thread_id
+        tid = pwndbg.aglib.proc.thread_id()
     print(
         message.notice(
             f"No tcache bins found for thread {message.hint(tid)} (the thread hasn't performed any allocations)."
@@ -164,7 +158,6 @@ def print_no_tcache_bins_found_error(tid: int | None = None) -> None:
 
 
 parser = argparse.ArgumentParser(
-    formatter_class=argparse.RawTextHelpFormatter,
     description="""Iteratively print chunks on a heap.
 
 Default to the current thread's active heap.""",
@@ -184,7 +177,7 @@ parser.add_argument(
 )
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.PTMALLOC2)
+@pwndbg.commands.Command(parser, category=CommandCategory.PTMALLOC2)
 @pwndbg.commands.OnlyWithResolvedHeapSyms
 @pwndbg.commands.OnlyWhenHeapIsInitialized
 @pwndbg.commands.OnlyWhenUserspace
@@ -213,7 +206,6 @@ def heap(addr: int | None = None, verbose: bool = False, simple: bool = False) -
 
 
 parser = argparse.ArgumentParser(
-    formatter_class=argparse.RawTextHelpFormatter,
     description="""Searches all heaps to find if an address belongs to a chunk. If yes, prints the chunk.""",
 )
 parser.add_argument(
@@ -235,7 +227,7 @@ parser.add_argument(
 )
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.PTMALLOC2)
+@pwndbg.commands.Command(parser, category=CommandCategory.PTMALLOC2)
 @pwndbg.commands.OnlyWhenRunning
 @pwndbg.commands.OnlyWithResolvedHeapSyms
 @pwndbg.commands.OnlyWhenHeapIsInitialized
@@ -267,7 +259,6 @@ def hi(addr: int, verbose: bool = False, simple: bool = False, fake: bool = Fals
 
 
 parser = argparse.ArgumentParser(
-    formatter_class=argparse.RawTextHelpFormatter,
     description="""Print the contents of an arena.
 
 Default to the current thread's arena.""",
@@ -275,7 +266,7 @@ Default to the current thread's arena.""",
 parser.add_argument("addr", nargs="?", type=int, default=None, help="Address of the arena.")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.PTMALLOC2)
+@pwndbg.commands.Command(parser, category=CommandCategory.PTMALLOC2)
 @pwndbg.commands.OnlyWithResolvedHeapSyms
 @pwndbg.commands.OnlyWhenHeapIsInitialized
 @pwndbg.commands.OnlyWhenUserspace
@@ -288,7 +279,7 @@ def arena(addr: int | None = None) -> None:
         arena = Arena(addr)
     else:
         arena = allocator.thread_arena
-        tid = pwndbg.aglib.proc.thread_id
+        tid = pwndbg.aglib.proc.thread_id()
         # arena might be None if the current thread doesn't allocate the arena
         if arena is None:
             print_no_arena_found_error(tid)
@@ -305,7 +296,7 @@ def arena(addr: int | None = None) -> None:
 parser = argparse.ArgumentParser(description="List this process's arenas.")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.PTMALLOC2)
+@pwndbg.commands.Command(parser, category=CommandCategory.PTMALLOC2)
 @pwndbg.commands.OnlyWithResolvedHeapSyms
 @pwndbg.commands.OnlyWhenHeapIsInitialized
 @pwndbg.commands.OnlyWhenUserspace
@@ -344,7 +335,7 @@ def arenas() -> None:
         ]
 
         for mapping_data in str(pwndbg.aglib.vmmap.find(first_heap.start)).split():
-            row.append(M.c.heap(mapping_data))
+            row.append(mem_color.c.heap(mapping_data))
 
         table.append(row)
 
@@ -356,7 +347,7 @@ def arenas() -> None:
             ]
 
             for mapping_data in str(pwndbg.aglib.vmmap.find(extra_heap.start)).split():
-                row.append(M.c.heap(mapping_data))
+                row.append(mem_color.c.heap(mapping_data))
 
             table.append(row)
 
@@ -364,7 +355,6 @@ def arenas() -> None:
 
 
 parser = argparse.ArgumentParser(
-    formatter_class=argparse.RawTextHelpFormatter,
     description="""Print a thread's tcache contents.
 
 Default to the current thread's tcache.""",
@@ -372,8 +362,9 @@ Default to the current thread's tcache.""",
 parser.add_argument("addr", nargs="?", type=int, default=None, help="Address of the tcache.")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.PTMALLOC2)
+@pwndbg.commands.Command(parser, category=CommandCategory.PTMALLOC2)
 @pwndbg.commands.OnlyWithResolvedHeapSyms
+@pwndbg.commands.OnlyWhenHeapIsInitialized
 @pwndbg.commands.OnlyWithTcache
 @pwndbg.commands.OnlyWhenUserspace
 def tcache(addr: int | None = None) -> None:
@@ -385,7 +376,7 @@ def tcache(addr: int | None = None) -> None:
 
     tcache = allocator.get_tcache(addr)
     # if the current thread doesn't allocate the arena, tcache will be NULL
-    tid = pwndbg.aglib.proc.thread_id
+    tid = pwndbg.aglib.proc.thread_id()
     if tcache:
         print(
             message.notice(
@@ -401,7 +392,7 @@ def tcache(addr: int | None = None) -> None:
 parser = argparse.ArgumentParser(description="Print the mp_ struct's contents.")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.PTMALLOC2)
+@pwndbg.commands.Command(parser, category=CommandCategory.PTMALLOC2)
 @pwndbg.commands.OnlyWithResolvedHeapSyms
 @pwndbg.commands.OnlyWhenHeapIsInitialized
 @pwndbg.commands.OnlyWhenUserspace
@@ -415,7 +406,6 @@ def mp() -> None:
 
 
 parser = argparse.ArgumentParser(
-    formatter_class=argparse.RawTextHelpFormatter,
     description="""Print relevant information about an arena's top chunk.
 
 Default to current thread's arena.""",
@@ -423,7 +413,7 @@ Default to current thread's arena.""",
 parser.add_argument("addr", nargs="?", type=int, default=None, help="Address of the arena.")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.PTMALLOC2)
+@pwndbg.commands.Command(parser, category=CommandCategory.PTMALLOC2)
 @pwndbg.commands.OnlyWithResolvedHeapSyms
 @pwndbg.commands.OnlyWhenHeapIsInitialized
 @pwndbg.commands.OnlyWhenUserspace
@@ -464,7 +454,7 @@ parser.add_argument(
 )
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.PTMALLOC2)
+@pwndbg.commands.Command(parser, category=CommandCategory.PTMALLOC2)
 @pwndbg.commands.OnlyWithResolvedHeapSyms
 @pwndbg.commands.OnlyWhenHeapIsInitialized
 @pwndbg.commands.OnlyWhenUserspace
@@ -482,9 +472,9 @@ def malloc_chunk(
 
     chunk = Chunk(addr)
 
-    headers_to_print: List[str] = []  # both state (free/allocated) and flags
-    fields_to_print: Set[str] = set()  # in addition to addr and size
-    out_fields = f"Addr: {M.get(chunk.address)}\n"
+    headers_to_print: list[str] = []  # both state (free/allocated) and flags
+    fields_to_print: set[str] = set()  # in addition to addr and size
+    out_fields = f"Addr: {mem_color.get(chunk.address)}\n"
 
     if fake:
         headers_to_print.append(message.on("Fake chunk"))
@@ -492,7 +482,7 @@ def malloc_chunk(
 
     if simple:
         if not headers_to_print:
-            headers_to_print.append(message.hint(M.get(chunk.address)))
+            headers_to_print.append(message.hint(mem_color.get(chunk.address)))
 
         out_fields = ""
         verbose = True
@@ -553,13 +543,13 @@ def malloc_chunk(
     print(" | ".join(headers_to_print) + "\n" + out_fields)
 
     if dump:
-        print(C.banner("hexdump"))
+        print(ctx_color.banner("hexdump"))
 
         ptr_size = pwndbg.aglib.arch.ptrsize
         pwndbg.commands.hexdump.hexdump(chunk.address, chunk.real_size + ptr_size)
 
     if next:
-        print(C.banner(f"Next {next} chunk(s):"))
+        print(ctx_color.banner(f"Next {next} chunk(s):"))
         for _ in range(next):
             chunk = chunk.next_chunk()
 
@@ -572,7 +562,6 @@ def malloc_chunk(
 
 
 parser = argparse.ArgumentParser(
-    formatter_class=argparse.RawTextHelpFormatter,
     description="""Print the contents of all an arena's bins and a thread's tcache.
 
 Default to the current thread's arena and tcache.""",
@@ -581,7 +570,7 @@ parser.add_argument("addr", nargs="?", type=int, default=None, help="Address of
 parser.add_argument("tcache_addr", nargs="?", type=int, default=None, help="Address of the tcache.")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.PTMALLOC2)
+@pwndbg.commands.Command(parser, category=CommandCategory.PTMALLOC2)
 @pwndbg.commands.OnlyWithResolvedHeapSyms
 @pwndbg.commands.OnlyWhenHeapIsInitialized
 @pwndbg.commands.OnlyWhenUserspace
@@ -607,7 +596,6 @@ def bins(addr: int | None = None, tcache_addr: int | None = None) -> None:
 
 
 parser = argparse.ArgumentParser(
-    formatter_class=argparse.RawTextHelpFormatter,
     description="""Print the contents of an arena's fastbins.
 
 Default to the current thread's arena.""",
@@ -618,7 +606,7 @@ parser.add_argument(
 )
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.PTMALLOC2)
+@pwndbg.commands.Command(parser, category=CommandCategory.PTMALLOC2)
 @pwndbg.commands.OnlyWithResolvedHeapSyms
 @pwndbg.commands.OnlyWhenHeapIsInitialized
 @pwndbg.commands.OnlyWhenUserspace
@@ -637,13 +625,12 @@ def fastbins(addr: int | None = None, verbose: bool = False) -> None:
 
     formatted_bins = format_bin(fastbins, verbose)
 
-    print(C.banner("fastbins"))
+    print(ctx_color.banner("fastbins"))
     for node in formatted_bins:
         print(node)
 
 
 parser = argparse.ArgumentParser(
-    formatter_class=argparse.RawTextHelpFormatter,
     description="""Print the contents of an arena's unsortedbin.
 
 Default to the current thread's arena.""",
@@ -654,7 +641,7 @@ parser.add_argument(
 )
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.PTMALLOC2)
+@pwndbg.commands.Command(parser, category=CommandCategory.PTMALLOC2)
 @pwndbg.commands.OnlyWithResolvedHeapSyms
 @pwndbg.commands.OnlyWhenHeapIsInitialized
 @pwndbg.commands.OnlyWhenUserspace
@@ -673,13 +660,12 @@ def unsortedbin(addr: int | None = None, verbose: bool = False) -> None:
 
     formatted_bins = format_bin(unsortedbin, verbose)
 
-    print(C.banner("unsortedbin"))
+    print(ctx_color.banner("unsortedbin"))
     for node in formatted_bins:
         print(node)
 
 
 parser = argparse.ArgumentParser(
-    formatter_class=argparse.RawTextHelpFormatter,
     description="""Print the contents of an arena's smallbins.
 
 Default to the current thread's arena.""",
@@ -690,7 +676,7 @@ parser.add_argument(
 )
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.PTMALLOC2)
+@pwndbg.commands.Command(parser, category=CommandCategory.PTMALLOC2)
 @pwndbg.commands.OnlyWithResolvedHeapSyms
 @pwndbg.commands.OnlyWhenHeapIsInitialized
 @pwndbg.commands.OnlyWhenUserspace
@@ -709,13 +695,12 @@ def smallbins(addr: int | None = None, verbose: bool = False) -> None:
 
     formatted_bins = format_bin(smallbins, verbose)
 
-    print(C.banner("smallbins"))
+    print(ctx_color.banner("smallbins"))
     for node in formatted_bins:
         print(node)
 
 
 parser = argparse.ArgumentParser(
-    formatter_class=argparse.RawTextHelpFormatter,
     description="""Print the contents of an arena's largebins.
 
 Default to the current thread's arena.""",
@@ -726,7 +711,7 @@ parser.add_argument(
 )
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.PTMALLOC2)
+@pwndbg.commands.Command(parser, category=CommandCategory.PTMALLOC2)
 @pwndbg.commands.OnlyWithResolvedHeapSyms
 @pwndbg.commands.OnlyWhenHeapIsInitialized
 @pwndbg.commands.OnlyWhenUserspace
@@ -744,13 +729,12 @@ def largebins(addr: int | None = None, verbose: bool = False) -> None:
 
     formatted_bins = format_bin(largebins, verbose)
 
-    print(C.banner("largebins"))
+    print(ctx_color.banner("largebins"))
     for node in formatted_bins:
         print(node)
 
 
 parser = argparse.ArgumentParser(
-    formatter_class=argparse.RawTextHelpFormatter,
     description="""Print the contents of a tcache.
 
 Default to the current thread's tcache.""",
@@ -761,8 +745,9 @@ parser.add_argument(
 )
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.PTMALLOC2)
+@pwndbg.commands.Command(parser, category=CommandCategory.PTMALLOC2)
 @pwndbg.commands.OnlyWithResolvedHeapSyms
+@pwndbg.commands.OnlyWhenHeapIsInitialized
 @pwndbg.commands.OnlyWithTcache
 @pwndbg.commands.OnlyWhenUserspace
 def tcachebins(addr: int | None = None, verbose: bool = False) -> None:
@@ -778,7 +763,7 @@ def tcachebins(addr: int | None = None, verbose: bool = False) -> None:
 
     formatted_bins = format_bin(tcachebins, verbose, offset=allocator.tcache_next_offset)
 
-    print(C.banner("tcachebins"))
+    print(ctx_color.banner("tcachebins"))
     for node in formatted_bins:
         print(node)
 
@@ -819,7 +804,7 @@ parser.add_argument(
 )
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.PTMALLOC2)
+@pwndbg.commands.Command(parser, category=CommandCategory.PTMALLOC2)
 @pwndbg.commands.OnlyWithResolvedHeapSyms
 @pwndbg.commands.OnlyWhenHeapIsInitialized
 @pwndbg.commands.OnlyWhenUserspace
@@ -921,7 +906,7 @@ def find_fake_fast(
 
     search_region = pwndbg.aglib.memory.read(search_start, search_end - search_start, partial=True)
 
-    print(C.banner("FAKE CHUNKS"))
+    print(ctx_color.banner("FAKE CHUNKS"))
     step = allocator.malloc_alignment if align else 1
     for i in range(0, len(search_region), step):
         candidate = search_region[i : i + size_field_width]
@@ -958,11 +943,18 @@ pwndbg.config.add_param(
     "default number of chunks to visualize",
 )
 
+pwndbg.config.add_param(
+    "vis-skip-repeating-val",
+    False,
+    "whether to skip repeating lines in vis command output",
+)
+
 parser = argparse.ArgumentParser(
-    formatter_class=argparse.RawTextHelpFormatter,
     description="""Visualize chunks on a heap.
 
-Default to the current arena's active heap.""",
+Default to the current arena's active heap.
+
+Repeated lines can be collapsed by setting 'vis-skip-repeating-val' config (on by default).""",
 )
 group = parser.add_mutually_exclusive_group()
 group.add_argument(
@@ -972,23 +964,36 @@ group.add_argument(
     default=pwndbg.config.default_visualize_chunk_number,
     help="Number of chunks to visualize. If the value is big enough and addr isn't provided, this is interpreted as addr instead.",
 )
-parser.add_argument("addr", nargs="?", default=None, help="Address of the first chunk.")
 parser.add_argument(
-    "--beyond_top",
+    "addr",
+    nargs="?",
+    default=None,
+    help="Address of the first chunk.",
+    type=int,
+)
+parser.add_argument(
+    "--beyond-top",
     "-b",
     action="store_true",
     default=False,
     help="Attempt to keep printing beyond the top chunk.",
 )
 parser.add_argument(
-    "--no_truncate",
+    "--no-skip",
+    "-s",
+    action="store_true",
+    default=False,
+    help="Don't skip repeating vals (Ignore the `visp-skip-repeating-val` configuration).",
+)
+parser.add_argument(
+    "--no-truncate",
     "-n",
     action="store_true",
     default=False,
     help="Display all the chunk contents (Ignore the `max-visualize-chunk-size` configuration).",
 )
 group.add_argument(
-    "--all_chunks",
+    "--all-chunks",
     "-a",
     action="store_true",
     default=False,
@@ -996,13 +1001,14 @@ group.add_argument(
 )
 
 
-@pwndbg.commands.ArgparsedCommand(parser, aliases=["vis"], category=CommandCategory.PTMALLOC2)
+@pwndbg.commands.Command(parser, aliases=["vis"], category=CommandCategory.PTMALLOC2)
 @pwndbg.commands.OnlyWithResolvedHeapSyms
 @pwndbg.commands.OnlyWhenHeapIsInitialized
 @pwndbg.commands.OnlyWhenUserspace
 def vis_heap_chunks(
     addr: int | None = None,
     count: int | None = None,
+    no_skip: bool = False,
     beyond_top: bool = False,
     no_truncate: bool = False,
     all_chunks: bool = False,
@@ -1123,7 +1129,22 @@ def vis_heap_chunks(
         pwndbg.lib.memory.round_up(int(pwndbg.config.max_visualize_chunk_size), ptr_size << 2) >> 1
     )
 
-    bin_labels_map: Dict[int, List[str]] = bin_labels_mapping(bin_collections)
+    bin_labels_map: dict[int, list[str]] = bin_labels_mapping(bin_collections)
+
+    # For collapsing repeated lines
+    skip_repeating: bool = False if no_skip else bool(pwndbg.config.vis_skip_repeating_val)
+    prev_line_content: str | None = None
+    repeat_count: int = 0
+    line_buffer: str = ""  # Temporary buffer for building current line (holds first cell)
+    saved_line_addr: str = ""  # Saved address for the current line
+
+    def flush_repeats() -> None:
+        """Add collapse message for accumulated repeated lines."""
+        nonlocal out, repeat_count, prev_line_content
+        if repeat_count > 0:
+            out += f"\n\t... ↓     {repeat_count:>3} repeated lines skipped"
+            repeat_count = 0
+        prev_line_content = None
 
     for c, stop in enumerate(chunk_delims):
         color_func = color_funcs[c % len(color_funcs)]
@@ -1135,6 +1156,10 @@ def vis_heap_chunks(
         begin_addr = pwndbg.lib.memory.round_down(cursor, ptr_size << 1)
         end_addr = pwndbg.lib.memory.round_down(stop, ptr_size << 1)
 
+        # Reset repeat tracking at chunk boundaries (only if skip_repeating is enabled)
+        if skip_repeating:
+            flush_repeats()
+
         while cursor != stop:
             # skip the middle part of a huge chunk
             if (
@@ -1149,13 +1174,15 @@ def vis_heap_chunks(
                 continue
 
             if printed % 2 == 0:
-                out += "\n0x%x" % cursor
+                saved_line_addr = f"0x{cursor:x}"
 
             data = pwndbg.aglib.memory.read(cursor, ptr_size)
             cell = pwndbg.aglib.arch.unpack(data)
             cell_hex = f"\t0x{cell:0{ptr_size * 2}x}"
 
-            out += color_func(cell_hex)
+            # Temporarily store colored cell_hex
+            colored_cell_hex = color_func(cell_hex)
+
             printed += 1
 
             labels.extend(bin_labels_map.get(cursor, []))
@@ -1163,18 +1190,75 @@ def vis_heap_chunks(
                 labels.append("Top chunk")
                 reached_top = True
 
+            # Build up the cell part (2 cells per line)
             asc += bin_ascii(data)
-            if printed % 2 == 0:
-                out += "\t" + color_func(asc) + ("\t <-- " + ", ".join(labels) if labels else "")
+
+            if printed % 2 == 1:
+                # First cell of the line, just accumulate
+                line_buffer += colored_cell_hex
+            else:
+                # Second cell - complete the line
+                line_label_part = "\t <-- " + ", ".join(labels) if labels else ""
+                colored_asc = color_func(asc)
+
+                # Build complete line content (address + cells + ascii + labels)
+                complete_line = (
+                    ("\n" if out else "")
+                    + saved_line_addr
+                    + line_buffer
+                    + colored_cell_hex
+                    + "\t"
+                    + colored_asc
+                    + line_label_part
+                )
+
+                if skip_repeating:
+                    # When skip_repeating is enabled, check for and collapse repeated lines
+                    # Don't collapse lines with labels (they're important markers)
+                    if not labels:
+                        # Compare just the hex values and ASCII part (exclude address and labels)
+                        current_hex_and_ascii = line_buffer + colored_cell_hex + "\t" + asc
+                        if prev_line_content == current_hex_and_ascii:
+                            # This line repeats the previous one, increment counter
+                            repeat_count += 1
+                        else:
+                            # Different line, flush any accumulated repeats and output this line
+                            flush_repeats()
+                            out += complete_line
+                            prev_line_content = current_hex_and_ascii
+                    else:
+                        # Line has labels, always output it
+                        flush_repeats()
+                        out += complete_line
+                        prev_line_content = None
+                else:
+                    # When skip_repeating is disabled, output every line directly
+                    out += complete_line
+
+                # Reset line building vars
+                line_buffer = ""
                 asc = ""
                 labels = []
 
             cursor += ptr_size
 
+    # Flush any remaining repeats (only matters if skip_repeating is enabled)
+    if skip_repeating:
+        flush_repeats()
+
     if printed % 2 != 0:
-        # Alignment whitespace of ("0x" + "00" * ptr_size) length.
+        # We have an incomplete line with only one cell
+        # Need to add the address, first cell, and padding
         machine_word_string_length = 2 + (2 * ptr_size)
-        out += "\t" + " " * machine_word_string_length + "\t" + color_func(asc)
+        out += (
+            ("\n" if out else "")
+            + saved_line_addr
+            + line_buffer
+            + "\t"
+            + " " * machine_word_string_length
+            + "\t"
+            + color_func(asc)
+        )
 
     print(out)
 
@@ -1195,17 +1279,17 @@ def vis_heap_chunks(
 VALID_CHARS = list(map(ord, set(printable) - set("\t\r\n\x0c\x0b")))
 
 
-def bin_ascii(bs):
+def bin_ascii(bs: bytes | bytearray | list[int]) -> str:
     return "".join(chr(c) if c in VALID_CHARS else "." for c in bs)
 
 
-def bin_labels_mapping(collections):
+def bin_labels_mapping(collections: list[Bins | None]) -> dict[int, list[str]]:
     """
     Returns all potential bin labels for all potential addresses
     We precompute all of them because doing this on demand was too slow and inefficient
     See #1675 for more details
     """
-    labels_mapping: Dict[int, List[str]] = {}
+    labels_mapping: dict[int, list[str]] = {}
 
     for bins in collections:
         if not bins:
@@ -1229,11 +1313,12 @@ def bin_labels_mapping(collections):
 try_free_parser = argparse.ArgumentParser(
     description="Check what would happen if free was called with given address."
 )
-try_free_parser.add_argument("addr", help="Address passed to free")
+try_free_parser.add_argument("addr", type=int, help="Address passed to free")
 
 
-@pwndbg.commands.ArgparsedCommand(try_free_parser, category=CommandCategory.PTMALLOC2)
+@pwndbg.commands.Command(try_free_parser, category=CommandCategory.PTMALLOC2)
 @pwndbg.commands.OnlyWhenHeapIsInitialized
+@pwndbg.commands.OnlyWithResolvedHeapSyms
 @pwndbg.commands.OnlyWhenUserspace
 def try_free(addr: str | int) -> None:
     addr = int(addr)
@@ -1241,7 +1326,7 @@ def try_free(addr: str | int) -> None:
     # check hook
     free_hook = pwndbg.aglib.symbol.lookup_symbol_addr("__free_hook")
     if free_hook is not None:
-        if pwndbg.aglib.memory.pvoid(free_hook) != 0:
+        if pwndbg.aglib.memory.read_pointer_width(free_hook) != 0:
             print(message.success("__libc_free: will execute __free_hook"))
 
     # free(0) has no effect
@@ -1266,7 +1351,7 @@ def try_free(addr: str | int) -> None:
 
     ptr_size = pwndbg.aglib.arch.ptrsize
 
-    def unsigned_size(size: int):
+    def unsigned_size(size: int) -> int:
         # read_chunk()['size'] is signed in pwndbg ;/
         # there may be better way to handle that
         if ptr_size < 8:
@@ -1274,7 +1359,7 @@ def try_free(addr: str | int) -> None:
         x = ctypes.c_uint64(size).value
         return x
 
-    def chunksize(chunk_size: int):
+    def chunksize(chunk_size: int) -> int:
         # maybe move this to ptmalloc.py
         return chunk_size & (~7)
 
@@ -1357,11 +1442,11 @@ def try_free(addr: str | int) -> None:
         and "key" in allocator.tcache_entry.keys()
     ):
         tc_idx = (chunk_size_unmasked - chunk_minsize + malloc_alignment - 1) // malloc_alignment
-        if allocator.mp is not None and tc_idx < int(allocator.mp["tcache_bins"]):
+        if allocator.mp is not None and tc_idx < allocator.tcache_small_bins:
             print(message.notice("Tcache checks"))
             e = addr + 2 * size_sz
             e += allocator.tcache_entry.keys().index("key") * ptr_size
-            e = pwndbg.aglib.memory.pvoid(e)
+            e = pwndbg.aglib.memory.read_pointer_width(e)
             tcache_addr = int(allocator.thread_cache.address)
             if e == tcache_addr:
                 # todo, actually do checks
@@ -1372,7 +1457,17 @@ def try_free(addr: str | int) -> None:
                 )
                 errors_found += 1
 
-            if int(allocator.get_tcache()["counts"][tc_idx]) < int(allocator.mp["tcache_count"]):
+            # May be an array, and tc_idx may be negative, so always cast to a
+            # pointer before we index into it.
+            # counts was renamed to num_slots in newer version of GLIBC 2.42
+            tcache = allocator.get_tcache()
+            try:
+                counts = tcache["num_slots"]
+            except Exception:
+                counts = tcache["counts"]
+            if int(counts.address.cast(counts.type.target().pointer())[tc_idx]) < int(
+                allocator.mp["tcache_count"]
+            ):
                 print(message.success("Using tcache_put"))
                 if errors_found == 0:
                     returned_before_error = True
@@ -1396,7 +1491,7 @@ def try_free(addr: str | int) -> None:
         except pwndbg.dbg_mod.Error as e:
             print(
                 message.error(
-                    f"Can't read next chunk at address 0x{chunk + chunk_size_unmasked:x}, memory error"
+                    f"Can't read next chunk at address 0x{addr + chunk_size_unmasked:x}, memory error"
                 )
             )
             finalize(errors_found, returned_before_error)
@@ -1592,24 +1687,3 @@ def try_free(addr: str | int) -> None:
 
 def try_unlink(addr: int) -> None:
     pass
-
-
-parser = argparse.ArgumentParser(description="Shows heap related configuration.")
-parser.add_argument(
-    "filter_pattern",
-    type=str,
-    nargs="?",
-    default=None,
-    help="Filter to apply to config parameters names/descriptions",
-)
-
-
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.PTMALLOC2)
-def heap_config(filter_pattern: str) -> None:
-    display_config(filter_pattern, "heap", has_file_command=False)
-
-    print(
-        message.hint(
-            "Some config values (e.g. main_arena) will be used only when resolve-heap-via-heuristic is `auto` or `force`"
-        )
-    )
