@@ -1,10 +1,10 @@
 from __future__ import annotations
 
 import argparse
-from typing import List
 
 import pwndbg
-import pwndbg.aglib.memory
+import pwndbg.aglib.strings
+import pwndbg.aglib.vmmap
 import pwndbg.commands
 from pwndbg.commands import CommandCategory
 
@@ -28,9 +28,9 @@ parser.add_argument(
 )
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.LINUX)
+@pwndbg.commands.Command(parser, category=CommandCategory.LINUX)
 @pwndbg.commands.OnlyWhenRunning
-def strings(n: int = 4, page_names: List[str] = [], save_as: str = None):
+def strings(n: int = 4, page_names: list[str] = [], save_as: str = None):
     # Get only readable pages and those that match the page_names filter
     pages = (
         p
