@@ -8,20 +8,20 @@ import argparse
 
 from tabulate import tabulate
 
+import pwndbg.aglib.dt
 import pwndbg.aglib.memory
 import pwndbg.aglib.symbol
 import pwndbg.aglib.tls
 import pwndbg.aglib.vmmap
-import pwndbg.color.memory as M
+import pwndbg.color
+import pwndbg.color.memory as mem_color
 import pwndbg.commands
 import pwndbg.commands.context
 import pwndbg.commands.telescope
-import pwndbg.dbg
 from pwndbg.color import message
 from pwndbg.commands import CommandCategory
 
 parser = argparse.ArgumentParser(
-    formatter_class=argparse.RawTextHelpFormatter,
     description="Print out base address of the current Thread Local Storage (TLS).",
 )
 
@@ -36,7 +36,7 @@ parser.add_argument(
 parser.add_argument("-a", "--all", action="store_true", help="Do not truncate the dump output.")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.LINUX)
+@pwndbg.commands.Command(parser, category=CommandCategory.LINUX)
 @pwndbg.commands.OnlyWhenRunning
 @pwndbg.commands.OnlyWhenUserspace
 def tls(pthread_self=False, all: bool = False) -> None:
@@ -46,7 +46,7 @@ def tls(pthread_self=False, all: bool = False) -> None:
         else pwndbg.aglib.tls.find_address_with_pthread_self()
     )
     if pwndbg.aglib.memory.is_readable_address(tls_base):
-        print(message.success("Thread Local Storage (TLS) base: %#x" % tls_base))
+        print(message.success(f"Thread Local Storage (TLS) base: {tls_base:#x}"))
         print(message.success("TLS is located at:"))
         print(message.notice(pwndbg.aglib.vmmap.find(tls_base)))
 
@@ -101,7 +101,6 @@ def tls(pthread_self=False, all: bool = False) -> None:
 
 
 parser = argparse.ArgumentParser(
-    formatter_class=argparse.RawTextHelpFormatter,
     description="List all threads belonging to the selected inferior.",
 )
 group = parser.add_mutually_exclusive_group()
@@ -123,7 +122,7 @@ group.add_argument(
 )
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.LINUX)
+@pwndbg.commands.Command(parser, category=CommandCategory.LINUX)
 @pwndbg.commands.OnlyWhenRunning
 @pwndbg.commands.OnlyWhenUserspace
 def threads(num_threads, respect_config) -> None:
@@ -181,7 +180,7 @@ def threads(num_threads, respect_config) -> None:
             thread.switch()
             pc = pwndbg.dbg.selected_frame().pc()
 
-            pc_colored = M.get(pc)
+            pc_colored = mem_color.get(pc)
             symbol = pwndbg.aglib.symbol.resolve_addr(pc)
 
             row.append(pc_colored)
