@@ -2,8 +2,9 @@ from __future__ import annotations
 
 import argparse
 
+import pwndbg.aglib.memory
 import pwndbg.chain
-import pwndbg.color as C
+import pwndbg.color as color
 import pwndbg.commands
 from pwndbg.commands import CommandCategory
 
@@ -12,7 +13,7 @@ parser.add_argument("addr", type=int, help="Address of the va_list")
 parser.add_argument("count", type=int, nargs="?", default=8, help="Number of arguments to dump")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.MISC)
+@pwndbg.commands.Command(parser, category=CommandCategory.MISC)
 @pwndbg.commands.OnlyWhenRunning
 def valist(addr: int, count: int) -> None:
     # The `va_list` struct looks like this:
@@ -33,7 +34,7 @@ def valist(addr: int, count: int) -> None:
     reg_save_area = pwndbg.aglib.memory.u64(addr + 16)
 
     indent = " " * len("gp_offset => ")
-    print(f"{C.blue('reg_save_area')}")
+    print(f"{color.blue('reg_save_area')}")
     for i in range(6):
         line = ""
         if i == gp_index:
@@ -45,6 +46,6 @@ def valist(addr: int, count: int) -> None:
         print(line)
 
     print()
-    print(f"{C.blue('overflow_arg_area')}")
+    print(f"{color.blue('overflow_arg_area')}")
     for i in range(count - 6):
         print(indent + pwndbg.chain.format(overflow_arg_area + i * 8))
