@@ -5,6 +5,7 @@ import argparse
 import pwndbg
 import pwndbg.aglib.memory
 import pwndbg.commands
+import pwndbg.dbg_mod
 from pwndbg.commands import CommandCategory
 
 
@@ -29,7 +30,7 @@ parser.add_argument("key", type=str, help="The key to use.")
 parser.add_argument("count", type=int, help="The number of bytes to xor.")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.MEMORY)
+@pwndbg.commands.Command(parser, category=CommandCategory.MEMORY)
 @pwndbg.commands.OnlyWhenRunning
 def xor(address, key, count) -> None:
     try:
@@ -44,7 +45,7 @@ parser.add_argument("address", type=int, help="The address to start xoring at.")
 parser.add_argument("count", type=int, help="The number of bytes to xor.")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.MEMORY)
+@pwndbg.commands.Command(parser, category=CommandCategory.MEMORY)
 @pwndbg.commands.OnlyWhenRunning
 def memfrob(address, count):
     return xor(address, "*", count)
