@@ -6,39 +6,44 @@ by using a decorator.
 
 from __future__ import annotations
 
+import functools
 import sys
 from collections import defaultdict
 from collections import deque
-from enum import Enum
-from enum import auto
+from collections.abc import Callable
 from functools import partial
 from functools import wraps
 from typing import Any
-from typing import Callable
-from typing import Deque
-from typing import Dict
-from typing import List
 from typing import TypeVar
 
 import gdb
 from typing_extensions import ParamSpec
 
+import pwndbg
+import pwndbg.lib.config
 from pwndbg import config
 from pwndbg.color import message
+from pwndbg.dbg_mod import EventHandlerPriority
+
+DISABLED = "disabled"
+DISABLED_DEADLOCK = "disabled-deadlock"
+ENABLED = "enabled"
 
-debug = config.add_param("debug-events", False, "display internal event debugging info")
 gdb_workaround_stop_event = config.add_param(
     "gdb-workaround-stop-event",
-    0,
-    "asynchronous stop events to improve 'commands' functionality.",
-    help_docstring="""
-Note: This may cause unexpected behavior with pwndbg or gdb.execute.
+    DISABLED,
+    "asynchronous stop events to improve 'commands' functionality",
+    help_docstring=f"""
+Note that this may cause unexpected behavior with Pwndbg or gdb.execute.
 
-Values:
-0 - Disable the workaround (default).
-1 - Enable asynchronous stop events; gdb.execute may behave unexpectedly(asynchronously).
-2 - Disable only deadlock detection; deadlocks may still occur.
+Values explained:
+
++ `{DISABLED}` - Disable the workaround (default).
++ `{DISABLED_DEADLOCK}` - Disable only deadlock detection; deadlocks may still occur.
++ `{ENABLED}` - Enable asynchronous stop events; gdb.execute may behave unexpectedly (asynchronously).
     """,
+    param_class=pwndbg.lib.config.PARAM_ENUM,
+    enum_sequence=[DISABLED, DISABLED_DEADLOCK, ENABLED],
 )
 
 P = ParamSpec("P")
@@ -60,7 +65,7 @@ T = TypeVar("T")
 # capture this so that we can fire off all of the 'start' events first.
 class StartEvent:
     def __init__(self) -> None:
-        self.registered: List[Callable[..., Any]] = []
+        self.registered: list[Callable[..., Any]] = []
         self.running = False
 
     def connect(self, function: Callable[..., Any]) -> None:
@@ -87,7 +92,9 @@ class StartEvent:
         self.on_new_objfile()
 
 
+# Monkeypatching GDB hihi
 gdb.events.start = StartEvent()
+gdb.events.suspend_all = object()
 
 
 def _is_safe_event_packet():
@@ -108,8 +115,9 @@ def _is_safe_event_thread():
     return True
 
 
-queued_events: Deque[Callable[..., Any]] = deque()
+queued_events: deque[Callable[..., Any]] = deque()
 executing_event = False
+workaround_thread_conn = None
 
 
 def _update_start_event_state(event_type: Any):
@@ -131,19 +139,22 @@ def _detect_deadlock():
         # Not executing an event inside another event, so no deadlock
         return
 
-    if gdb_workaround_stop_event == 2:
+    if gdb_workaround_stop_event == DISABLED_DEADLOCK:
         # Skip deadlock detection because this option disables it
         return
 
     print(message.error("DEADLOCK DETECTED..."))
     print(
         message.error(
-            f"""The deadlock issue is likely caused by using 'commands[\\n]{message.hint("continue")}[\\n]end'.
+            f"""The deadlock issue is likely caused by using 'commands[\\n]{
+                message.hint("continue")
+            }[\\n]end'.
 
 To address this, you have three options:
 
 1. Avoid using '{message.hint("commands")}'. Instead, rewrite it as a Python script. For example:
-   {message.hint('''
+   {
+                message.hint('''
    # Read more at: https://github.com/pwndbg/pwndbg/issues/425#issuecomment-892302716
    class Bp(gdb.Breakpoint):
        def stop(self):
@@ -151,19 +162,22 @@ To address this, you have three options:
            return False  # False = continue to next breakpoint, True = stop inferior
 
    Bp("main")
-   ''')}
+   ''')
+            }
 
-2. Replace '{message.hint("continue")}' with '{message.hint('pi gdb.execute("continue")')}' and use '{message.hint("set gdb-workaround-stop-event 2")}'.
+2. Replace '{message.hint("continue")}' with '{
+                message.hint('pi gdb.execute("continue")')
+            }' and use '{message.hint("set gdb-workaround-stop-event disabled-deadlock")}'.
    This change reduces the likelihood of deadlocks, while preserving pwndbg functionality.
 
-3. Run '{message.hint("set gdb-workaround-stop-event 1")}', allowing you to keep '{message.hint("continue")}' as is.
+3. Run '{message.hint("set gdb-workaround-stop-event enabled")}', allowing you to keep '{
+                message.hint("continue")
+            }' as is.
    However, this setting may cause pwndbg or gdb.execute to behave asynchronously/unpredictably.
 """
         )
     )
-    import os
-
-    os._exit(1)
+    sys.exit(1)
 
 
 def wrap_safe_event_handler(event_handler: Callable[P, T], event_type: Any) -> Callable[P, T]:
@@ -209,16 +223,29 @@ def wrap_safe_event_handler(event_handler: Callable[P, T], event_type: Any) -> C
             # https://github.com/pwndbg/pwndbg/issues/2576
             gdb.post_event(_loop_until_thread_ok)
             return
-        elif event_type == gdb.events.stop:
+        if event_type == gdb.events.stop:
             # Workaround to issue with gdb `commands \n continue \n end` - Selected thread is running
             # https://github.com/pwndbg/pwndbg/issues/425
-            if gdb_workaround_stop_event == 1:
+            if gdb_workaround_stop_event == ENABLED:
                 gdb.post_event(_loop_until_thread_ok)
                 return
 
             executing_event = True
             gdb.execute("", to_string=True)  # Trigger bug in gdb, it is like 'yield'
             executing_event = False
+        elif event_type in (gdb.events.cont, gdb.events.new_thread):
+            # Workaround for crash in gdb when used: `target extended-remote` + `attach`
+            # https://github.com/pwndbg/pwndbg/issues/3231
+            global workaround_thread_conn
+            conn = gdb.selected_inferior().connection
+            if (
+                isinstance(conn, gdb.RemoteTargetConnection)
+                and conn.type == "extended-remote"
+                and conn.is_valid()
+                and workaround_thread_conn != conn
+            ):
+                gdb.selected_inferior().threads()[0].switch()
+                workaround_thread_conn = conn
 
         while queued_events:
             queued_events.popleft()()
@@ -226,18 +253,9 @@ def wrap_safe_event_handler(event_handler: Callable[P, T], event_type: Any) -> C
     return _inner_handler
 
 
-class HandlerPriority(Enum):
-    """
-    A priority level for an event handler, ordered from highest to lowest priority.
-    """
-
-    CACHE_CLEAR = auto()
-    LOW = auto()
-
-
 # In order to support reloading, we must be able to re-fire
 # all 'objfile' and 'stop' events.
-registered: Dict[Any, Dict[HandlerPriority, List[Callable[..., Any]]]] = {
+registered: dict[gdb.EventRegistry[Any], dict[EventHandlerPriority, list[Callable[..., None]]]] = {
     gdb.events.exited: {},
     gdb.events.cont: {},
     gdb.events.new_objfile: {},
@@ -249,102 +267,78 @@ registered: Dict[Any, Dict[HandlerPriority, List[Callable[..., Any]]]] = {
     gdb.events.register_changed: {},
 }
 
-# Registered events are wrapped and aren't directly connected to GDB
-# This is a map from event to the actual handler connected to GDB
-connected = {}
-
 # Keys are gdb.events.*
 paused = defaultdict(bool)
 
 
-def pause(event_registry) -> None:
+def pause(event_registry: gdb.EventRegistry[Any]) -> None:
     paused[event_registry] = True
 
 
-def unpause(event_registry) -> None:
+def unpause(event_registry: gdb.EventRegistry[Any]) -> None:
     paused[event_registry] = False
 
 
-def connect(
-    func: Callable[[], T],
-    event_handler: Any,
-    name: str = "",
-    priority: HandlerPriority = HandlerPriority.LOW,
-) -> Callable[[], T]:
-    if debug:
-        print("Connecting", func.__name__, event_handler)
+def event_handler_factory(
+    event_registry: gdb.EventRegistry[Any], event_name: str, priority: EventHandlerPriority
+) -> Callable[[Callable[..., None]], Callable[..., None]]:
+    """
+    Essentially the implementation of pwndbg.dbg_mod.gdb.event_handler().
 
-    @wraps(func)
-    def caller(*a: P.args, **kw: P.kwargs) -> None:
-        if paused[event_handler]:
-            return None
+    Takes a gdb.EventRegistry because that contains some events that pwndbg.dbg_mod.EventType
+    doesn't.
 
-        if debug:
-            sys.stdout.write(f"{name!r} {func.__module__}.{func.__name__} {a!r}\n")
+    Read the first few paragraphs of:
+    https://sourceware.org/gdb/current/onlinedocs/gdb.html/Events-In-Python.html#Events-In-Python
+    """
 
-        try:
-            # Don't pass the event along to the decorated function.
-            # This is because there are functions with multiple event decorators
-            func()
-        except Exception as e:
-            import pwndbg.exception
+    def decorator(fn: Callable[..., None]) -> Callable[..., None]:
+        # This is only executed once.
 
-            pwndbg.exception.handle()
-            raise e
+        if pwndbg.config.dev_debug_events:
+            print("Connecting", fn.__name__, event_name)
 
-    registered[event_handler].setdefault(priority, []).append(caller)
-    if event_handler not in connected:
-        handle = partial(invoke_event, event_handler)
-        handle = wrap_safe_event_handler(handle, event_handler)
+        should_connect: bool = False
 
-        event_handler.connect(handle)
-        connected[event_handler] = handle
-    return func
+        if not registered[event_registry]:
+            # We've never seen this type of event before.
+            should_connect = True
 
+        # Wrap the function for dev instrumentation
+        @functools.wraps(fn)
+        def _dev_wrapper(*a, **kw) -> None:
+            if pwndbg.config.dev_debug_events:
+                sys.stdout.write(
+                    f"{event_name} ({priority.name}) {fn.__module__}.{fn.__qualname__}\n"
+                )
 
-def exit(func: Callable[[], T], **kwargs: Any) -> Callable[[], T]:
-    return connect(func, gdb.events.exited, "exit", **kwargs)
+            return fn(*a, **kw)
 
+        # Register this event handler with us.
+        registered[event_registry].setdefault(priority, []).append(_dev_wrapper)
 
-def cont(func: Callable[[], T], **kwargs: Any) -> Callable[[], T]:
-    return connect(func, gdb.events.cont, "cont", **kwargs)
+        if should_connect:
+            # We actually need to tell gdb to run `invoke_event` when
+            # the appropriate event is invoked. Then inside `invoke_event`
+            # we will call all `registered` functions.
 
+            wrapped_invoke_event = partial(invoke_event, event_registry)
+            wrapped_invoke_event = wrap_safe_event_handler(wrapped_invoke_event, event_registry)
 
-def new_objfile(func: Callable[[], T], **kwargs: Any) -> Callable[[], T]:
-    return connect(func, gdb.events.new_objfile, "obj", **kwargs)
+            event_registry.connect(wrapped_invoke_event)
 
+        return _dev_wrapper
 
-def stop(func: Callable[[], T], **kwargs: Any) -> Callable[[], T]:
-    return connect(func, gdb.events.stop, "stop", **kwargs)
-
-
-def start(func: Callable[[], T], **kwargs: Any) -> Callable[[], T]:
-    return connect(func, gdb.events.start, "start", **kwargs)
-
-
-def thread(func: Callable[[], T], **kwargs: Any) -> Callable[[], T]:
-    return connect(func, gdb.events.new_thread, "thread", **kwargs)
-
-
-def before_prompt(func: Callable[[], T], **kwargs: Any) -> Callable[[], T]:
-    return connect(func, gdb.events.before_prompt, "before_prompt", **kwargs)
-
-
-def reg_changed(func: Callable[[], T], **kwargs: Any) -> Callable[[], T]:
-    return connect(func, gdb.events.register_changed, "reg_changed", **kwargs)
-
-
-def mem_changed(func: Callable[[], T], **kwargs: Any) -> Callable[[], T]:
-    return connect(func, gdb.events.memory_changed, "mem_changed", **kwargs)
+    return decorator
 
 
 def log_objfiles(ofile: gdb.NewObjFileEvent | None = None) -> None:
-    if not (debug and ofile):
+    if not (pwndbg.config.dev_debug_events and ofile):
         return None
 
     name = ofile.new_objfile.filename
 
-    print("objfile: %r" % name)
+    print(f"objfile: {name!r}")
     gdb.execute("info sharedlibrary")
     return None
 
@@ -352,13 +346,28 @@ def log_objfiles(ofile: gdb.NewObjFileEvent | None = None) -> None:
 gdb.events.new_objfile.connect(log_objfiles)
 
 
-# invoke all registered handlers of a certain event type
-def invoke_event(event: Any, *args: Any, **kwargs: Any) -> None:
-    handlers = registered.get(event)
-    if handlers is not None:
-        for prio in HandlerPriority:
-            for f in handlers.get(prio, []):
-                f(*args, **kwargs)
+def invoke_event(event_registry: gdb.EventRegistry[Any], event_data: Any = None) -> None:
+    """
+    Invoke all registered handlers for a certain event_registry.
+    event_data may be None if we manually ran this function.
+    """
+    if paused[event_registry] or paused[gdb.events.suspend_all]:
+        return
+
+    handlers = registered.get(event_registry)
+    if handlers is None:
+        return
+
+    try:
+        # Run all the event handlers for this event from lowest to highest priority.
+        for prio in EventHandlerPriority:
+            for func in handlers.get(prio, []):
+                func()
+    except Exception as e:
+        import pwndbg.exception
+
+        pwndbg.exception.handle()
+        raise e
 
 
 def after_reload(fire_start: bool = True) -> None:
