@@ -1,7 +1,7 @@
 from __future__ import annotations
 
+from collections.abc import Iterator
 from contextlib import contextmanager
-from typing import Iterator
 
 import gdb
 
@@ -22,8 +22,10 @@ def lock_scheduler() -> Iterator[None]:
     old_config = gdb.parameter("scheduler-locking")
     if old_config != "on":
         gdb.execute("set scheduler-locking on")
-        yield
-        gdb.execute(f"set scheduler-locking {old_config}")
+        try:
+            yield
+        finally:
+            gdb.execute(f"set scheduler-locking {old_config}")
     else:
         yield
 
