@@ -1,38 +1,36 @@
 from __future__ import annotations
 
 from collections import defaultdict
+from collections.abc import Callable
+from collections.abc import Sequence
+from enum import Enum
 from functools import total_ordering
 from typing import Any
-from typing import Callable
-from typing import DefaultDict
-from typing import Dict
-from typing import List
-from typing import Sequence
 from typing import TypeVar
 
 T = TypeVar("T")
 
 # Boolean value. True or False, same as in Python.
 PARAM_BOOLEAN = 0
+# Boolean value, or 'auto'.
+PARAM_AUTO_BOOLEAN = 1
+# Signed integer value. Disallows zero.
+PARAM_INTEGER = 2
 # Signed integer value.
-PARAM_ZINTEGER = 1
-# String value. Accepts escape sequences.
-PARAM_STRING = 2
+PARAM_ZINTEGER = 3
+# Unsigned integer value. Disallows zero.
+PARAM_UINTEGER = 4
 # Unsigned integer value.
-PARAM_ZUINTEGER = 3
+PARAM_ZUINTEGER = 5
+# Unlimited ZUINTEGER.
+PARAM_ZUINTEGER_UNLIMITED = 6
+# String value. Accepts escape sequences.
+PARAM_STRING = 7
 # String value, accepts only one of a number of possible values, specified at
 # parameter creation.
-PARAM_ENUM = 4
+PARAM_ENUM = 8
 # String value corresponding to the name of a file, if present.
-PARAM_OPTIONAL_FILENAME = 5
-# Boolean value, or 'auto'.
-PARAM_AUTO_BOOLEAN = 6
-# Unlimited ZUINTEGER.
-PARAM_ZUINTEGER_UNLIMITED = 7
-# Signed integer value. Disallows zero.
-PARAM_INTEGER = 8
-# Unsigned integer value. Disallows zero.
-PARAM_UINTEGER = 9
+PARAM_OPTIONAL_FILENAME = 9
 
 PARAM_CLASSES = {
     # The Python boolean values, True and False are the only valid values.
@@ -45,6 +43,18 @@ PARAM_CLASSES = {
     str: PARAM_STRING,
 }
 
+# Strings that will be used in supplementing the parameter help_docstring
+HELP_DEFAULT_PREFIX = "Default:"
+HELP_VALID_VALUES_PREFIX = "Valid values:"
+
+
+class Scope(Enum):
+    # If you want to add another scope here, don't forget to add
+    # a command which prints it!
+    config = 1
+    theme = 2
+    heap = 3
+
 
 # @total_ordering allows us to implement `__eq__` and `__lt__` and have all the
 # other comparison operators handled for us
@@ -59,32 +69,36 @@ class Parameter:
         help_docstring: str = "",
         param_class: int | None = None,
         enum_sequence: Sequence[str] | None = None,
-        scope: str = "config",
+        scope: Scope = Scope.config,
     ) -> None:
         self.name = name
         self.default = default
         self._value = default
         self.param_class = param_class or PARAM_CLASSES[type(default)]
-        # Note: `set_show_doc` should be a noun phrase, e.g. "the value of the foo"
-        # The `set_doc` will be "Set the value of the foo."
-        # The `show_doc` will be "Show the value of the foo."
-        # `get_set_string()` will return "Set the value of the foo to VALUE."
-        # `get_show_string()` will return "Show the value of the foo."
-        self.set_show_doc = set_show_doc.strip()
-        self.help_docstring = help_docstring.strip()
-        # Show the default value in the parameter help
-        self.help_docstring += "\n\nDefault: " + self.pretty_default()
+        # Note: If `set_show_doc` is "the value of foo" then:
+        # The `set_doc` will be "Set the value of foo."
+        # The `show_doc` will be "Show the value of foo."
+        # `get_set_string()` will return "Set the value of foo to VALUE."
+        # `get_show_string()` will return "Show the value of foo."
+        self.set_show_doc = set_show_doc
+        self.help_docstring = help_docstring
+        # Show the default value in the parameter help.
+        # We add a trailing double space because docs are in markdown.
+        self.help_docstring += "\n\n" + HELP_DEFAULT_PREFIX + " " + self.pretty_default() + "  "
         # Show valid values if they aren't obvious
         if param_class == PARAM_ENUM:
             self.help_docstring += (
-                "\nValid values: " + ", ".join([f"'{name}'" for name in enum_sequence]) + "."
+                "\n"
+                + HELP_VALID_VALUES_PREFIX
+                + " "
+                + ", ".join([f"'{name}'" for name in enum_sequence])
             )
         if param_class == PARAM_AUTO_BOOLEAN:
-            self.help_docstring += "\nValid values: on, off, auto."
+            self.help_docstring += "\n" + HELP_VALID_VALUES_PREFIX + " on, off, auto."
 
         self.enum_sequence = enum_sequence
         self.scope = scope
-        self.update_listeners: List[Callable[[Any], None]] = []
+        self.update_listeners: list[Callable[[Any], None]] = []
 
     def add_update_listener(self, listener: Callable[[Any], None]) -> None:
         self.update_listeners.append(listener)
@@ -118,15 +132,17 @@ class Parameter:
         """Convert a value this object could contain to its pretty string representation."""
         if self.param_class == PARAM_BOOLEAN:
             return "on" if val else "off"
-        elif self.param_class == PARAM_AUTO_BOOLEAN:
+        if self.param_class == PARAM_AUTO_BOOLEAN:
             if val is None:
                 return "auto"
-            else:
-                return "on" if val else "off"
-        elif self.param_class == PARAM_STRING or self.param_class == PARAM_ENUM:
+            return "on" if val else "off"
+        if (
+            self.param_class == PARAM_STRING
+            or self.param_class == PARAM_ENUM
+            or self.param_class == PARAM_OPTIONAL_FILENAME
+        ):
             return "'" + val + "'"
-        else:
-            return str(val)
+        return str(val)
 
     def pretty(self) -> str:
         return self.pretty_val(self.value)
@@ -197,8 +213,8 @@ class Parameter:
 
 class Config:
     def __init__(self) -> None:
-        self.params: Dict[str, Parameter] = {}
-        self.triggers: DefaultDict[str, List[Callable[..., Any]]] = defaultdict(list)
+        self.params: dict[str, Parameter] = {}
+        self.triggers: defaultdict[str, list[Callable[..., Any]]] = defaultdict(list)
 
     def add_param(
         self,
@@ -209,18 +225,37 @@ class Config:
         help_docstring: str = "",
         param_class: int | None = None,
         enum_sequence: Sequence[str] | None = None,
-        scope: str = "config",
+        scope: Scope = Scope.config,
     ) -> Parameter:
         # Dictionary keys are going to have underscores, so we can't allow them here
         assert "_" not in name
-        assert len(name) <= 32 and "Config name too long."
+        assert len(name) <= 32 and "Parameter name too long."
+        assert name and "Parameter name cannot be empty."
+
+        set_show_doc = set_show_doc.strip()
+        help_docstring = help_docstring.strip()
+        assert set_show_doc and "Parameter's set_show_doc cannot be empty."
         assert (
             len(set_show_doc) <= 70
-            and "Config set_show_doc too long, use the help_docstring parameter."
+            and "Parameter's set_show_doc too long, use the help_docstring parameter."
         )
+        assert set_show_doc[-1] != "." and "Don't end set_show_doc with punctuation."
+        assert (
+            HELP_DEFAULT_PREFIX not in help_docstring
+            and f"Having the string '{HELP_DEFAULT_PREFIX}' in the help_docstring "
+            "messes with documentation generation. Please remove it, it is automatically generated."
+        )
+        assert (
+            HELP_VALID_VALUES_PREFIX not in help_docstring
+            and f"Having the string '{HELP_VALID_VALUES_PREFIX}' in the help_docstring "
+            "messes with documentation generation. Please remove it, you can use param_class=PARAM_ENUM."
+        )
+
         if param_class == PARAM_ENUM or enum_sequence:
             assert param_class == PARAM_ENUM and enum_sequence
 
+        assert scope in Scope
+
         p = Parameter(
             name,
             default,
@@ -251,14 +286,13 @@ class Config:
 
         return wrapper
 
-    def get_params(self, scope: str) -> List[Parameter]:
+    def get_params(self, scope: Scope) -> list[Parameter]:
         return sorted(filter(lambda p: p.scope == scope, self.params.values()))
 
     def __getattr__(self, name: str) -> Parameter:
         if name in self.params:
             return self.params[name]
-        else:
-            raise AttributeError(f"'Config' object has no attribute '{name}'")
+        raise AttributeError(f"'Config' object has no attribute '{name}'")
 
     def __setattr__(self, attr, val):
         if attr in ("params", "triggers"):
