@@ -7,9 +7,7 @@ from __future__ import annotations
 
 import binascii
 import socket
-from typing import List
-
-import pwndbg.aglib.arch
+import struct
 
 # http://students.mimuw.edu.pl/lxr/source/include/net/tcp_states.h
 TCP_STATUSES = {
@@ -27,6 +25,12 @@ TCP_STATUSES = {
 }
 
 
+def format_host_port(ip, port):
+    if ":" in ip and not ip.startswith("["):
+        return f"[{ip}]:{port}"
+    return f"{ip}:{port}"
+
+
 class inode:
     inode: int | None = None
 
@@ -44,9 +48,7 @@ class Connection(inode):
     family: str | None = None
 
     def __str__(self) -> str:
-        return (
-            f"{self.family} {self.lhost}:{self.lport} => {self.rhost}:{self.rport} ({self.status})"
-        )
+        return f"{self.family} {format_host_port(self.lhost, self.lport)} => {format_host_port(self.rhost, self.rport)} ({self.status})"
 
     def __repr__(self) -> str:
         return f'Connection("{self}")'
@@ -56,13 +58,13 @@ class UnixSocket(inode):
     path = "(anonymous)"
 
     def __str__(self) -> str:
-        return "unix %r" % self.path
+        return f"unix {self.path!r}"
 
     def __repr__(self) -> str:
         return f"UnixSocket({self})"
 
 
-def tcp(data: str) -> List[Connection]:
+def _tcp_parser(data: str, ip_family: socket.AddressFamily, endianness: str) -> list[Connection]:
     # For reference, see:
     # https://www.kernel.org/doc/Documentation/networking/proc_net_tcp.txt
     """
@@ -73,7 +75,7 @@ def tcp(data: str) -> List[Connection]:
     if not data:
         return []
 
-    result: List[Connection] = []
+    result: list[Connection] = []
     for line in data.splitlines()[1:]:
         fields = line.split()
         """
@@ -119,10 +121,19 @@ def tcp(data: str) -> List[Connection]:
             host, port = hostport.split(":")
             host = binascii.unhexlify(host)
 
-            if pwndbg.aglib.arch.endian == "little":
-                host = host[::-1]
+            if endianness == "little":
+                if ip_family == socket.AF_INET:
+                    words = struct.unpack("<1I", host)
+                    host = struct.pack(">1I", *words)
+                elif ip_family == socket.AF_INET6:
+                    # The kernel outputs the IPv6 address as 4 little-endian 32-bit chunks.
+                    # This behavior is specific to little-endian kernels, such as x86.
+                    # On a big-endian kernel, the byte order would differ.
+                    # Reference: https://github.com/torvalds/linux/blob/a7c428ee8f59f171a3b57474f2bd5cee0ef1e036/net/ipv6/tcp_ipv6.c#L2153
+                    words = struct.unpack("<4I", host)
+                    host = struct.pack(">4I", *words)
 
-            host = socket.inet_ntop(socket.AF_INET, host)
+            host = socket.inet_ntop(ip_family, host)
             port = int(port, 16)
             return host, port
 
@@ -138,11 +149,19 @@ def tcp(data: str) -> List[Connection]:
     return result
 
 
-def unix(data: str) -> List[UnixSocket]:
+def tcp(data: str, endianness: str) -> list[Connection]:
+    return _tcp_parser(data, socket.AF_INET, endianness)
+
+
+def tcp6(data: str, endianness: str) -> list[Connection]:
+    return _tcp_parser(data, socket.AF_INET6, endianness)
+
+
+def unix(data: str) -> list[UnixSocket]:
     if not data:
         return []
 
-    result: List[UnixSocket] = []
+    result: list[UnixSocket] = []
     # Note: it is super important to split by "\n" instead of .splitlines() here
     # because there may be a line like this:
     # "0000000000000000: 00000002 00000000 00000000 0002 01 23302 @@@@\x9e\x05@@\x01=\r@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n"
@@ -191,7 +210,7 @@ NETLINK_TYPES = {
 
 class Netlink(inode):
     eth: int = 0
-    pid: int | None = None
+    portid: int | None = None
 
     def __str__(self) -> str:
         return NETLINK_TYPES.get(self.eth, "(unknown netlink)")
@@ -200,18 +219,18 @@ class Netlink(inode):
         return f"Netlink({self})"
 
 
-def netlink(data: str) -> List[Netlink]:
+def netlink(data: str) -> list[Netlink]:
     if not data:
         return []
 
-    result: List[Netlink] = []
+    result: list[Netlink] = []
     for line in data.splitlines()[1:]:
         # sk       Eth Pid    Groups   Rmem     Wmem     Dump     Locks     Drops     Inode            [10/8747]
         fields = line.split()
 
         n = Netlink()
         n.eth = int(fields[1])
-        n.pid = int(fields[2])
+        n.portid = int(fields[2])  # 'Pid' in Netlink context refers to Port ID, not Process ID
         n.inode = int(fields[9])
         result.append(n)
 
