@@ -1,13 +1,13 @@
 from __future__ import annotations
 
 import re
+from enum import Enum
 from random import choice
-from typing import List
 
 from pwndbg.color import message
 
 # GDB specific tips
-GDB_TIPS: List[str] = [
+GDB_TIPS: list[str] = [
     "GDB's `apropos <topic>` command displays all registered commands that are related to the given <topic>",
     "GDB's `follow-fork-mode` parameter can be used to set whether to trace parent or child after fork() calls. Pwndbg sets it to child by default",
     'Use GDB\'s `dprintf` command to print all calls to given function. E.g. `dprintf malloc, "malloc(%p)\\n", (void*)$rdi` will print all malloc calls',
@@ -16,15 +16,16 @@ GDB_TIPS: List[str] = [
     "If you have debugging symbols the `info args` command shows current frame's function arguments (use `up` and `down` to switch between frames)",
     'Calling functions like `call (void)puts("hello world")` will run all other target threads for the time the function runs. Use `set scheduler-locking on` to lock the execution to current thread when calling functions',
     "Use the `pipe <cmd> | <prog>` command to pass output of a GDB/Pwndbg command to a shell program, e.g. `pipe elfsections | grep bss`. This can also be shortened to: `| <cmd> | <prog>`",
+    "Prefixing a command with `!` in GDB will execute it as a shell command, e.g.: `!ls` or `!cat flag.txt`",
 ]
 
 # Pwndbg specific tips
-PWNDBG_TIPS: List[str] = [
+PWNDBG_TIPS: list[str] = [
     "If you want Pwndbg to clear screen on each command (but still save previous output in history) use `set context-clear-screen on`",
     "The `set show-flags on` setting will display CPU flags register in the regs context panel",
     "GDB and Pwndbg parameters can be shown or set with `show <param>` and `set <param> <value>` GDB commands",
     "Use Pwndbg's `config` and `theme` commands to tune its configuration and theme colors!",
-    "Pwndbg mirrors some of Windbg commands like `eq`, `ew`, `ed`, `eb`, `es`, `dq`, `dw`, `dd`, `db`, `ds` for writing and reading memory",
+    "Pwndbg mirrors some of WinDbg commands like `eq`, `ew`, `ed`, `eb`, `es`, `dq`, `dw`, `dd`, `db`, `ds` for writing and reading memory",
     "Pwndbg resolves kernel memory maps by parsing page tables (default) or via `monitor info mem` QEMU gdbstub command (use `set kernel-vmmap-via-page-tables off` for that)",
     "Use the `vmmap` command for a better & colored memory maps display (than the GDB's `info proc mappings`)",
     "Use the `telescope` command to dereference a given address/pointer multiple times (if the dereferenced value is a valid ptr; see `config telescope` to configure its behavior)",
@@ -33,7 +34,7 @@ PWNDBG_TIPS: List[str] = [
     "Pwndbg context displays where the program branches to thanks to emulating few instructions into the future. You can disable this with `set emulate off` which may also speed up debugging",
     "Use the `canary` command to see all stack canary/cookie values on the stack (based on the *usual* stack canary value initialized by glibc)",
     "Use the `procinfo` command for better process introspection (than the GDB's `info proc` command)",
-    "Want to display each context panel in a separate tmux window? See https://github.com/pwndbg/pwndbg/blob/dev/FEATURES.md#splitting--layouting-context",
+    "Want to display each context panel in a separate tmux window? See https://pwndbg.re/stable/tutorials/splitting-the-context/",
     'Use `$base("heap")` to get the start address of a [heap] memory page',
     "Use the `errno` (or `errno <number>`) command to see the name of the last or provided (libc) error",
     "Pwndbg sets the SIGLARM, SIGBUS, SIGPIPE and SIGSEGV signals so they are not passed to the app; see `info signals` for full GDB signals configuration",
@@ -55,7 +56,7 @@ PWNDBG_TIPS: List[str] = [
 ]
 
 # LLDB specific tips
-LLDB_TIPS: List[str] = [
+LLDB_TIPS: list[str] = [
     "Use LLDB's `help <command>` to get detailed help on any command",
     "LLDB's `expr` command lets you evaluate expressions in the current frame context",
     "Use `frame variable` (or `fr v`) to show all variables in the current frame",
@@ -79,25 +80,37 @@ LLDB_TIPS: List[str] = [
 ]
 
 
-def get_tip_of_the_day() -> str:
+def get_tip_of_the_day(debugger_type: int) -> str:
     """
     Returns a random tip based on the current debugger type.
+
+    You should pass in pwndbg.dbg.name().value .
     """
-    return choice(get_all_tips())
+    return choice(get_all_tips(debugger_type))
 
 
-def get_all_tips() -> List[str]:
+def get_all_tips(debugger_type: int) -> list[str]:
     """
     Returns all tips applicable to the current debugger.
-    """
-    import pwndbg.dbg
 
-    if pwndbg.dbg.name() == pwndbg.dbg_mod.DebuggerType.GDB:
-        return GDB_TIPS + PWNDBG_TIPS
-    elif pwndbg.dbg.name() == pwndbg.dbg_mod.DebuggerType.LLDB:
-        return LLDB_TIPS + PWNDBG_TIPS
-    else:
-        return PWNDBG_TIPS
+    You should pass in pwndbg.dbg.name().value .
+    """
+
+    # Needs to mirror pwndbg/dbg_mod/__init__.py:DebuggerType
+    class DebuggerType(Enum):
+        GDB = 1
+        LLDB = 2
+
+    valid_values = [e.value for e in DebuggerType]
+    assert debugger_type in valid_values
+
+    match debugger_type:
+        case DebuggerType.GDB.value:
+            return GDB_TIPS + PWNDBG_TIPS
+        case DebuggerType.LLDB.value:
+            return LLDB_TIPS + PWNDBG_TIPS
+        case _:
+            return PWNDBG_TIPS
 
 
 def color_tip(tip: str) -> str:
