@@ -9,8 +9,8 @@ def build_id() -> str:
     Returns pwndbg commit id if git is available.
     """
     pwndbg_dir = os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
-    # If we install pwndbg into site-packages, then `gdbinit.py` is missing.
-    if not os.path.exists(os.path.join(pwndbg_dir, "gdbinit.py")):
+    # If we install pwndbg into site-packages, then `.pwndbg_root` is missing.
+    if not os.path.exists(os.path.join(pwndbg_dir, ".pwndbg_root")):
         return ""
 
     try:
@@ -19,7 +19,7 @@ def build_id() -> str:
 
         commit_id = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
 
-        return "build: %s" % commit_id.decode("utf-8").strip("\n")
+        return "build: {}".format(commit_id.decode("utf-8").strip("\n"))
 
     except (OSError, subprocess.CalledProcessError):
         # OSError -> no git in $PATH
@@ -27,7 +27,7 @@ def build_id() -> str:
         return ""
 
 
-__version__ = "2025.04.13"
+__version__ = "2026.02.18"
 
 b_id = build_id()
 
