@@ -1,8 +1,12 @@
+"""
+Logging.
+"""
+
 from __future__ import annotations
 
 import logging
 
-import pwndbg.color
+import pwndbg.color.message
 
 
 class ColorFormatter(logging.Formatter):
